@@ -5,15 +5,19 @@
 /*                       Copyright (c) 1995,1996                         */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -37,24 +41,38 @@
 #include <iostream.h>
 #include <string.h>
 #include <math.h>
+#include <fcntl.h>
 #include "EST_system.h"
 #include "EST_socket.h"
 #include "EST_Option.h"
 #include "EST_Wave.h"
 #include "EST_io_aux.h"
 #include "audioP.h"
-#include "EST_play_audio.h"
+#include "EST_audio.h"
+#include "EST_wave_aux.h"
 
 static int play_sunau_wave(EST_Wave &inwave, EST_Option &al);
 static int play_socket_wave(EST_Wave &inwave, EST_Option &al);
 static int play_aucomm_wave(EST_Wave &inwave, EST_Option &al);
 
+static int record_sunau_wave(EST_Wave &wave, EST_Option &al);
+
 int play_wave(EST_Wave &inwave, EST_Option &al)
 {
     EST_String protocol;
+    EST_Wave wtmp;
+    EST_Wave *toplay;
     char *quality;
     char *sr;
 
+    if (inwave.num_channels() > 1)
+    {
+	wave_combine_channels(wtmp,inwave);
+	toplay = &wtmp;
+    }
+    else
+	toplay = &inwave;
+
     if ((sr = getenv("NA_PLAY_HOST")) != NULL)
 	if  (!al.present("-display"))
 	    al.add_item("-display", sr);
@@ -71,6 +89,8 @@ int play_wave(EST_Wave &inwave, EST_Option &al)
     {
 	if (nas_supported)
 	    protocol = "netaudio";  // the default protocol
+	else if (esd_supported)
+	    protocol = "esdaudio";
 	else if (sun16_supported)
 	    protocol = "sun16audio";
 	else if (freebsd16_supported)
@@ -88,24 +108,26 @@ int play_wave(EST_Wave &inwave, EST_Option &al)
     }
 
     if (upcase(protocol) == "NETAUDIO")
-	return play_na_wave(inwave,al);
+	return play_nas_wave(*toplay,al);
+    else if (upcase(protocol) == "ESDAUDIO")
+	return play_esd_wave(*toplay,al);
     else if (upcase(protocol) == "SUNAUDIO")
-	return play_sunau_wave(inwave,al);
+	return play_sunau_wave(*toplay,al);
     else if (upcase(protocol) == "SUN16AUDIO")
-	return play_sun16_wave(inwave,al);
+	return play_sun16_wave(*toplay,al);
     else if ((upcase(protocol) == "FREEBSD16AUDIO") ||
 	     (upcase(protocol) == "LINUX16AUDIO"))
-	return play_voxware_wave(inwave,al);
+	return play_voxware_wave(*toplay,al);
     else if (upcase(protocol) == "IRIXAUDIO")
-	return play_irix_wave(inwave,al);
+	return play_irix_wave(*toplay,al);
     else if (upcase(protocol) == "MPLAYERAUDIO")
-	return play_mplayer_wave(inwave,al);
+	return play_mplayer_wave(*toplay,al);
     else if (upcase(protocol) == "WIN32AUDIO")
-	return play_win32audio_wave(inwave,al);
+	return play_win32audio_wave(*toplay,al);
     else if (upcase(protocol) == "AUDIO_COMMAND")
-	return play_aucomm_wave(inwave,al);
+	return play_aucomm_wave(*toplay,al);
     else if (upcase(protocol) == "SOCKET")
-	return play_socket_wave(inwave,al);
+	return play_socket_wave(*toplay,al);
     else
     {
 	cerr << "Unknown audio server protocol " << protocol << endl;
@@ -131,7 +153,7 @@ static int play_socket_wave(EST_Wave &inwave, EST_Option &al)
     if (al.present("socket_otype"))
 	otype = al.val("socket_otype");  // file type to send to client
     else
-	otype = "nist";
+	otype = "riff";
     
     inwave.save(tmpfile,otype);
     
@@ -191,13 +213,18 @@ static int play_sunau_wave(EST_Wave &inwave, EST_Option &al)
     // Play wave through /dev/audio using 8K ulaw encoding
     // works for Suns as well as Linux and FreeBSD machines
     int rcode;
-    (void)al; // currently unused parameter
+    char *audiodevice;
 
     inwave.resample(8000);
 
+    if (al.present("-audiodevice"))
+	audiodevice = al.val("-audiodevice");
+    else
+	audiodevice = "/dev/audio";
+
     // Should really do something cute about checking if /dev/audio
     // is not in use
-    rcode = inwave.save("/dev/audio","ulaw");
+    rcode = inwave.save(audiodevice,"ulaw");
 
     return rcode;
 
@@ -213,6 +240,8 @@ EST_String options_supported_audio(void)
     audios += " audio_command";
     if (nas_supported)
 	audios += " netaudio";
+    else if (esd_supported)
+	audios += " esdaudio";
     if (sun16_supported)
 	audios += " sun16audio";
     if (freebsd16_supported)
@@ -225,6 +254,117 @@ EST_String options_supported_audio(void)
 	audios += " mplayeraudio";
     if (win32audio_supported)
 	audios += " win32audio";
+    if (os2audio_supported)
+	audios += " os2audio";
 
     return audios;
 }
+
+int record_wave(EST_Wave &wave, EST_Option &al)
+{
+    // Record wave from audio device 
+    char *sr;
+    EST_String protocol;
+
+    // For archaic reasons, if you are using NAS use DISPLAY or
+    // AUDIOSERVER 
+    if ((sr = getenv("NA_PLAY_HOST")) != NULL)
+	if  (!al.present("-display"))
+	    al.add_item("-display", sr);
+
+    if (al.present("-p"))
+	protocol = al.val("-p");
+    else if ((sr=getenv("NA_PLAY_PROTOCOL")) != NULL)
+	protocol = sr;
+    else if (protocol == "")
+    {
+	if (nas_supported)
+	    protocol = "netaudio";  // the default protocol
+	else if (esd_supported)
+	    protocol = "esdaudio";  // the default protocol
+	else if (sun16_supported)
+	    protocol = "sun16audio";
+	else if (freebsd16_supported)
+	    protocol = "freebsd16audio";
+	else if (linux16_supported)
+	    protocol = "linux16audio";
+	else if (irix_supported)
+	    protocol = "irixaudio";
+	else if (win32audio_supported)
+	    protocol = "win32audio";
+	else if (mplayer_supported)
+	    protocol = "mplayeraudio";
+	else
+	    protocol = "sunaudio";
+    }
+
+    if (upcase(protocol) == "NETAUDIO")
+	return record_nas_wave(wave,al);
+    else if (upcase(protocol) == "ESDAUDIO")
+        return record_esd_wave(wave,al);
+    else if (upcase(protocol) == "SUN16AUDIO")
+	return record_sun16_wave(wave,al);
+    else if ((upcase(protocol) == "FREEBSD16AUDIO") ||
+	     (upcase(protocol) == "LINUX16AUDIO"))
+	return record_voxware_wave(wave,al);
+    else if (upcase(protocol) == "SUNAUDIO")
+	return record_sunau_wave(wave,al);
+    else
+    {
+	cerr << "NA_RECORD: \"" << protocol << 
+	    "\" EST current has no record support" << endl;
+	return -1;
+    }
+}
+
+static int record_sunau_wave(EST_Wave &wave, EST_Option &al)
+{
+    int num_samples,i,r,n;
+    int audio;
+    unsigned char *ulawwave;
+    short *waveform;
+    const int AUDIOBUFFSIZE = 256;
+    char *audiodevice;
+
+    if (al.present("-audiodevice"))
+	audiodevice = al.val("-audiodevice");
+    else
+	audiodevice = "/dev/audio";
+
+    if ((audio = open(audiodevice, O_RDONLY)) == -1)
+    {
+	cerr << "SUN16: can't open " << audiodevice << " for reading" << endl;
+	return -1;
+    }
+
+    num_samples = (int)(8000*al.fval("-time"));
+    ulawwave = walloc(unsigned char,num_samples);
+    
+    for (r=i=0; i < num_samples; i+= r)
+    {
+	if (num_samples > i+AUDIOBUFFSIZE)
+	    n = AUDIOBUFFSIZE;
+	else
+	    n = num_samples-i;
+	r = read(audio,&ulawwave[i], n);
+	if (r <= 0)
+	{
+	    cerr << "sunaudio: failed to read from audio device" << endl;
+	    close(audio);
+	    wfree(ulawwave);
+	    return -1;
+	}
+    }
+
+    wave.resize(num_samples);
+    wave.set_sample_rate(8000);
+    waveform = wave.values().memory();
+
+    ulaw_to_short(ulawwave,waveform,num_samples);
+    wave.resample(al.ival("-sample_rate"));
+
+    close(audio);
+    wfree(ulawwave);
+    return 0;
+}
+
