@@ -5,15 +5,19 @@
  /*                      Copyright (c) 1995,1996                          */
  /*                        All Rights Reserved.                           */
  /*                                                                       */
- /*  Permission to use, copy, modify, distribute this software and its    */
- /*  documentation for research, educational and individual use only, is  */
- /*  hereby granted without fee, subject to the following conditions:     */
+ /*  Permission is hereby granted, free of charge, to use and distribute  */
+ /*  this software and its documentation without restriction, including   */
+ /*  without limitation the rights to use, copy, modify, merge, publish,  */
+ /*  distribute, sublicense, and/or sell copies of this work, and to      */
+ /*  permit persons to whom this work is furnished to do so, subject to   */
+ /*  the following conditions:                                            */
  /*   1. The code must retain the above copyright notice, this list of    */
  /*      conditions and the following disclaimer.                         */
  /*   2. Any modifications must be clearly marked as such.                */
  /*   3. Original authors' names are not deleted.                         */
- /*  This software may not be used for commercial purposes without        */
- /*  specific prior written permission from the authors.                  */
+ /*   4. The authors' names are not used to endorse or promote products   */
+ /*      derived from this software without specific prior written        */
+ /*      permission.                                                      */
  /*                                                                       */
  /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
  /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -40,6 +44,7 @@
 #include "EST_TVector.h"
 #include "EST_matrix_support.h"
 #include "EST_cutils.h"
+#include "EST_error.h"
 
 template<class T>
 void EST_TVector<T>::default_vals()
@@ -78,13 +83,7 @@ EST_TVector<T>::EST_TVector(int n,
 {
   default_vals();
 
-  p_memory = memory;
-  p_offset = offset;
-  p_sub_matrix = !free_when_destroyed;
-
-  p_num_columns = n;
-
-  p_column_step =1;
+  set_memory(memory, offset, n, free_when_destroyed);
 }
 
 template<class T>
@@ -96,7 +95,7 @@ EST_TVector<T>::~EST_TVector()
 
   if (p_memory != NULL && !p_sub_matrix)
     {
-      delete [] p_memory;
+      delete [] (p_memory-p_offset);
       p_memory = NULL;
     }
 }
@@ -106,7 +105,7 @@ template<class T>
 void  EST_TVector<T>::fill(const T &v)
 {
     for (int i = 0; i < num_columns(); ++i)
-	fast_a(i) = v;
+	fast_a_v(i) = v;
 }
 
 template<class T>
@@ -114,9 +113,9 @@ void EST_TVector<T>::set_memory(T *buffer, int offset, int columns,
 				int free_when_destroyed)
 {
   if (p_memory != NULL && !p_sub_matrix)
-    delete [] p_memory;
+    delete [] (p_memory-p_offset);
   
-  p_memory = buffer;
+  p_memory = buffer-offset;
   p_offset=offset;
   p_num_columns = columns;
   p_column_step=1;
@@ -148,7 +147,7 @@ void EST_TVector<T>::get_values(T *data,
 template<class T>
 void EST_TVector<T>::copy_data(const EST_TVector<T> &a)
 {
-  set_values(a.p_memory + a.p_offset, a.p_column_step, 0, num_columns());
+  set_values(a.p_memory, a.p_column_step, 0, num_columns());
 }
 
 template<class T>
@@ -163,19 +162,14 @@ void EST_TVector<T>::just_resize(int new_cols, T** old_vals)
 {
   T *new_m;
   
-  if (num_columns() != new_cols)
+  if (num_columns() != new_cols || p_memory == NULL )
     {
       if (p_sub_matrix)
-	{
-	  cerr << "Error: attempt to resize Sub-Vector\n";
-	  return;
-	}
+	EST_error("Attempt to resize Sub-Vector");
 
       if (new_cols < 0)
-	{
-	  cerr << "Error: attempt to resize vector to negative size: " << new_cols << "\n";
-	  return;
-	}
+	EST_error("Attempt to resize vector to negative size: %d",
+		  new_cols);
 
       new_m = new T[new_cols];
 
@@ -183,13 +177,16 @@ void EST_TVector<T>::just_resize(int new_cols, T** old_vals)
 	if (old_vals != NULL)
 	  *old_vals = p_memory;
 	else if (!p_sub_matrix)
-	  delete [] p_memory;
+	  delete [] (p_memory-p_offset);
 
       p_memory = new_m;
+      //cout << "vr: mem: " << p_memory << " (" << (int)p_memory << ")\n";
       p_offset=0;
       p_num_columns = new_cols;
       p_column_step=1;
     }
+  else
+    *old_vals = p_memory;
 }
 
 
@@ -208,14 +205,15 @@ void EST_TVector<T>::resize(int new_cols, int set)
     {
       int copy_c = 0;
 
-      if (old_vals != p_memory)
+      if (!old_vals)
+	copy_c=0;
+      else if (old_vals != p_memory)
 	{
 	  copy_c = Lof(num_columns(), old_cols);
 
 	  for(i=0; i<copy_c; i++)
 	      a_no_check(i) 
 		= old_vals[vcell_pos(i,
-				    old_offset,
 				    old_column_step)];
 	}
       else 
@@ -226,7 +224,7 @@ void EST_TVector<T>::resize(int new_cols, int set)
     }
 
   if (old_vals && old_vals != p_memory && !p_sub_matrix)
-    delete [] old_vals;
+    delete [] (old_vals-old_offset);
 }
 
 template<class T>
@@ -240,14 +238,10 @@ EST_write_status EST_TVector<T>::save(const EST_String &file)
 	outf = new ofstream(file);
     
     if (!(*outf))
-    {
-	cerr << "TVector: can't open file \"" << file <<"\" for writing"
-	    << endl;
-	return misc_write_error;
-    }
+      return misc_write_error;
     
     for (i = 0; i < n(); ++i)
-      *outf << fast_a(i) << "\t";
+      *outf << fast_a_v(i) << "\t";
     *outf << endl;
     
     if (outf != &cout)
@@ -268,7 +262,7 @@ T &EST_TVector<T>::a_check(int n)
   if (!EST_vector_bounds_check(n, num_columns(), FALSE))
     return *error_return;
 
-  return fast_a(n);
+  return fast_a_v(n);
 }
 
 template<class T>
@@ -284,23 +278,11 @@ int EST_TVector<T>::operator == (const EST_TVector<T> &v) const
 	return 0;
 
     for(int i=0; i<num_columns() ; i++)
-      if (fast_a(i) != v.fast_a(i))
+      if (fast_a_v(i) != v.fast_a_v(i))
 	return 0;
     return 1;
 }
 
-template<class T>
-int EST_TVector<T>::operator !=(const EST_TVector<T> &v) const
-{
-    if (num_columns() != v.num_columns())
-	return 1;
-
-    for(int i=0; i<num_columns() ; i++)
-	if (fast_a(i) != v.fast_a(i))
-	    return 1;
-    return 0;
-}
-
 template<class T>
 void EST_TVector<T>::copy_section(T* dest, int offset, int num) const
 {
@@ -336,11 +318,11 @@ void EST_TVector<T>::sub_vector(EST_TVector<T> &sv,
     len = num_columns()-start_c;
 
   if (sv.p_memory != NULL && ! sv.p_sub_matrix)
-    delete [] sv.p_memory;
+    delete [] (sv.p_memory - sv.p_offset);
 
   sv.p_sub_matrix = TRUE;
-  sv.p_memory = p_memory;
   sv.p_offset = p_offset + start_c*p_column_step;
+  sv.p_memory = p_memory - p_offset + sv.p_offset;
   sv.p_column_step=p_column_step;
   sv.p_num_columns = len;
 }
@@ -354,3 +336,14 @@ int i;
  return st;
 }
 
+template<class T>
+void EST_TVector<T>::integrity() const
+{
+  cout << "integrity: p_memory=" << p_memory << endl;
+  if(p_memory == (T *)0x00080102) 
+    {
+      cout << "fatal value!!!\n";
+    }
+}
+
+
