@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -137,17 +141,17 @@ bool EST_BackoffNgrammarState::accumulate(const EST_StrVector &words,
 					  const double count)
 {
 
-    int i;
-    cerr << "accumulate ";
-    for(i=0;i<words.n();i++)
-    {
-	if(words.n()-1-p_level == i)
-	    cerr <<"*";
-	cerr << words(i);
-	if(words.n()-1-p_level == i)
-	    cerr <<"*";
-    }
-    cerr << endl;
+//    int i;
+//    cerr << "accumulate ";
+//    for(i=0;i<words.n();i++)
+//    {
+//	if(words.n()-1-p_level == i)
+//	    cerr <<"*";
+//	cerr << words(i);
+//	if(words.n()-1-p_level == i)
+//	    cerr <<"*";
+//    }
+//    cerr << endl;
 
     // update pdf at this node
     p_pdf.cumulate(words(words.n()-1-p_level),count);
@@ -176,18 +180,18 @@ bool EST_BackoffNgrammarState::accumulate(const EST_IVector &words,
 					  const double count)
 {
 
-    int i;
-    cerr << "accumulate level " << p_level << " : ";
-    for(i=0;i<words.n();i++)
-    {
-	if(words.n()-1-p_level == i)
-	    cerr <<"*";
-	cerr << p_pdf.item_name(words(i));
-	if(words.n()-1-p_level == i)
-	    cerr <<"*";
-	cerr << " ";
-    }
-    cerr << endl;
+//    int i;
+//    cerr << "accumulate level " << p_level << " : ";
+//    for(i=0;i<words.n();i++)
+//    {
+//	if(words.n()-1-p_level == i)
+//	    cerr <<"*";
+//	cerr << p_pdf.item_name(words(i));
+//	if(words.n()-1-p_level == i)
+//	    cerr <<"*";
+//	cerr << " ";
+//    }
+//    cerr << endl;
 
     // update pdf at this node
     p_pdf.cumulate(words(words.n()-1-p_level),count);
@@ -593,7 +597,8 @@ bool EST_Ngrammar::init_sparse_representation()
 	return false;
     }
     
-    p_states = new EST_NgrammarState[(int)pow(vocab->length(),p_order-1)];
+    p_num_states = (int)pow(vocab->length(),p_order-1);
+    p_states = new EST_NgrammarState[p_num_states];
     
     return (bool)(p_states != NULL);
 }
@@ -792,7 +797,7 @@ bool EST_Ngrammar::build(const EST_StrList &filenames,
 
     bool skip_this;
     EST_String new_filename;
-    EST_TBI *p;
+    EST_Litem *p;
     for (p = filenames.head(); p; p = next(p))
     {
 	cerr << "Building from " << filenames(p) << endl;
@@ -1227,7 +1232,8 @@ bool EST_Ngrammar::build_ngram(const EST_String &filename,
 	else
 	    bad_word=0;
 
-	cerr << "at start : bad word at " << bad_word << endl;
+	if(bad_word > 0)
+	  cerr << "at start : bad word at " << bad_word << endl;
 
     }
     while (!ts.eof())
@@ -1254,7 +1260,8 @@ bool EST_Ngrammar::build_ngram(const EST_String &filename,
 		    accumulate(window);
 		else
 		{
-		    cerr << "not accumulating : bad word at " << bad_word << endl;
+		    cerr << "not accumulating : bad word at " << bad_word;
+		    cerr << " window=" << window; // l<< endl;
 		    bad_word--;
 		}
 	    }
@@ -1316,7 +1323,7 @@ bool EST_Ngrammar::build_ngram(const EST_String &filename,
 		// check for bad tags
 		if (window(p_order-1) == -1)
 		    bad_word = p_order;
-		else if(window(p_order-2) == -1)
+		else if( (p_order>1) && (window(p_order-2) == -1) )
 		    bad_word = p_order-1;
 		else
 		    bad_word=0;
@@ -1682,6 +1689,8 @@ int EST_Ngrammar::find_next_state_id(int state, int word) const
     // Find a new state index from the current after moving with word
     int i,f;
 
+    if (p_order == 1)
+	return 0;
     for (f=1,i=0; i<p_order-2; i++)
 	f*=vocab->length();
     return ((state%f)*vocab->length())+word;
@@ -2205,226 +2214,13 @@ EST_Ngrammar::save(const EST_String &filename, const EST_String type,
 	return save_ngram_cstr_ascii(filename, *this,trace,floor);
     if (type == "cstr_bin")
 	return save_ngram_cstr_bin(filename, *this, trace,floor);
+    if (type == "wfst")
+	return save_ngram_wfst(filename, *this);
     
     cerr << "EST_Ngrammar::save unknown output file type " << type << endl;
     return write_fail;
 }
 
-
-bool
-EST_Ngrammar::stats(const EST_String filename,
-		    double &raw_entropy, double &count,
-		    double &entropy, double &perplexity,
-		    const EST_String &prev,
-		    const EST_String &prev_prev,
-		    const EST_String &last,
-		    const EST_String &input_format) const
-{
-    double H;
-    int Q;
-    double prob;
-    //EST_StrVector window(p_order);
-    EST_StrVector window(p_order);
-    EST_TokenStream ts;
-    int bad_word=0;
-    EST_String s;
-    int eoln_is_eos = TRUE;
-    int sliding_window = TRUE;
-    int wcount=0;
-
-    (void)last;
-    
-    // H = -1/Q . log P(w1,w1,w2,...wQ) ]
-    // to do : should log be base 2 ??
-    
-    if (filename == "-")
-    {
-	if( ts.open(stdin, FALSE) == -1)
-	{
-	    cerr << "EST_Ngrammar:: test failed to open stdin";
-	    cerr << " for reading" << endl;
-	    return false;
-	}
-    }
-    else if (ts.open(filename) == -1)
-    {
-	cerr << "EST_Ngrammar:: test failed to open \"" << filename 
-	    << "\" for reading" << endl;
-	return false;
-    }
-    
-    if ((input_format == "sentence_per_line") || (input_format == ""))
-    {
-	eoln_is_eos = TRUE;
-	sliding_window = TRUE;
-    }
-    else if (input_format == "sentence_per_file")
-    {
-	eoln_is_eos = FALSE;
-	sliding_window = TRUE;
-    }
-    else if(input_format == "ngram_per_line")
-    {
-	eoln_is_eos = FALSE;
-	sliding_window = FALSE;
-    }
-    else
-    {
-	cerr << "EST_Ngrammar:: test file unknown input format \"" << 
-	    input_format << "\" for reading" << endl;
-	return false;
-    }
-    
-    H=0.0;
-    Q=0;
-    
-    if(sliding_window)
-    {
-	fill_window_start(window,prev,prev_prev);
-	// check for bad tags
-	if (wordlist_index(prev) == -1)
-	    bad_word = p_order;
-	else if( (p_order>1) && (wordlist_index(prev_prev) == -1))
-	    bad_word = p_order-1;
-	else
-	    bad_word=0;
-    }
-    
-    for(Q=0; !ts.eof(); )
-    {
-	s=ts.get().string();
-	
-	if (s!="")
-	{
-	    if(sliding_window)
-	    {
-		slide(window,-1);
-		window[p_order-1] = s;
-		if (wordlist_index(s) < 0)
-		{
-		    cerr << "EST_Ngrammar::stats " <<
-			" word \"" << s << "\" is not in vocabulary, skipping"
-			    << endl;
-		    bad_word = p_order;
-		}
-	    
-		if (bad_word == 0)
-		{
-		    prob = probability(window); //,false,true);
-		    H += log(prob);
-		    Q++;
-		
-		    //cerr << Q << " " << H << window << endl;
-		}
-		else
-		    bad_word--;
-	    }
-	    else
-	    {
-		// not a sliding window - wait for end of line and accumulate
-		if(wcount < p_order)
-		{
-		    window[wcount++] = s;
-
-		    if ( ((wcount == p_order) && predlist_index(s) < 0)
-			|| ((wcount != p_order) && wordlist_index(s) < 0)  )
-		    {
-			cerr << "EST_Ngrammar::stats " <<
-			    " word \"" << s << "\" is not in vocabulary, skipping"
-				<< endl;
-			bad_word = 1;
-		    }
-		}
-		else
-		    cerr << "Too many items on line  - ignoring trailing ones !" << endl;
-	    }
-
-	    
-	}
-	
-	
-	// end of sentence ?
-	if(ts.eoln())
-	{
-	    if(!sliding_window)
-	    {
-		if((wcount == p_order) && bad_word == 0)
-		{
-		    prob = probability(window); //,false,true);
-		    H += log(prob);
-		    Q++;
-		}
-		wcount = 0;
-		bad_word = 0;
-	    }
-	    else if (eoln_is_eos)
-	    {		
-	    
-		slide(window,-1);
-		window[p_order-1] = last;
-	    
-		if(wordlist_index(last) == -1)
-		{
-		    //cerr << "WARNING : skipping over bad word '" 
-		    //<< last << "'" << endl;
-		    bad_word = p_order;
-		}
-		
-		if (bad_word == 0)
-		{
-		    prob = probability(window); //,false,true);
-		    H += log(prob);
-		    Q++;
-		}
-		
-		fill_window_start(window,prev,prev_prev);
-		
-		// check for bad tags
-		if (wordlist_index(prev) == -1)
-		    bad_word = p_order;
-		else if( (p_order>1) && (wordlist_index(prev_prev) == -1))
-		    bad_word = p_order-1;
-		else
-		    bad_word=0;
-	    }
-	}
-	
-    }
-    // if last accumulate was at end of sentence
-    // we don't need to do the extra accumulate
-    if (sliding_window && (window(p_order-1) != prev))
-    {
-	
-	// and finish with the ngram [.....last few words,last]
-	slide(window,-1);
-	window[p_order-1] = last;
-	
-	if (wordlist_index(last) == -1)
-	    bad_word=p_order;
-	
-	if (bad_word == 0)
-	{
-	    prob = probability(window); //,false,true);
-	    H += log(prob);
-	    Q++;
-	    
-	}
-    }
-    
-    H = -1.0 * H;
-    
-    // convert to log base 2
-    H /= log(2.0);
-    
-    count = Q;
-    raw_entropy = H;
-    entropy = H/Q;
-    perplexity = pow(2.0,entropy);
-    
-    return true;
-}    
-
-
 void EST_Ngrammar::iterate(EST_StrVector &words,
 			   void (*function)(EST_Ngrammar *n,
 					    EST_StrVector &words,
