@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1996,1997                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -75,7 +79,7 @@ load_ngram_arpa(const EST_String filename, EST_Ngrammar &n, const EST_StrList &v
     if (ts.open(filename) == -1)
 	return misc_read_error;
 
-    // find  \\data\\  
+    // find  backslash data backslash
     while ((!ts.eof()) && !ts.get().string().contains("\\data\\"));
 
     if (ts.eof())
@@ -195,7 +199,7 @@ load_ngram_arpa(const EST_String filename, EST_Ngrammar &n, const EST_StrList &v
     } // loop through orders
     
 
-    // find \\end\\   
+    // find backslash end backslash 
     while (!ts.eof())
 	if (ts.get().string() == "\\end\\")
 	{
@@ -355,7 +359,7 @@ load_ngram_cstr_bin(const EST_String filename, EST_Ngrammar &n)
     if (swap)
 	swap_bytes_double(dd,num_entries);
     
-    for(j=i=0;i<n.states();i++)
+    for(j=i=0;i<n.num_states();i++)
     {
 	if (j >= num_entries)
 	{
@@ -781,6 +785,44 @@ save_ngram_cstr_ascii(const EST_String filename, EST_Ngrammar &n,
     return write_ok;
 }
 
+EST_write_status 
+save_ngram_wfst(const EST_String filename, EST_Ngrammar &n)
+{
+    // Save as a WFST
+    FILE *ost;
+    int i;
+
+    if ((ost = fopen(filename,"wb")) == NULL)
+    {
+	cerr << "Ngrammar save: unable to open \"" << filename << 
+	    "\" for writing" << endl;
+	return write_fail;
+    }
+
+    fprintf(ost,"EST_File fst\n");
+    fprintf(ost,"DataType ascii\n");
+    fprintf(ost,"in \"(");
+    for (i=0; i < n.vocab->length(); i++)
+	fprintf(ost," %s\n",(const char *)n.vocab->name(i));
+    fprintf(ost," )\"\n");
+    fprintf(ost,"out \"(");
+    for (i=0; i < n.vocab->length(); i++)
+	fprintf(ost," %s\n",(const char *)n.vocab->name(i));
+    fprintf(ost," )\"\n");
+    fprintf(ost,"NumStates %d\n",n.num_states());
+    fprintf(ost,"EST_Header_End\n");
+
+    for (i=0; i<n.num_states(); i++)
+    {
+	fprintf(ost,"((%d nonfinal %d)\n",i,i);
+	fprintf(ost,")\n");
+    }
+
+    fclose(ost);
+    
+    return write_ok;
+}
+
 EST_write_status 
 save_ngram_cstr_bin(const EST_String filename, EST_Ngrammar &n, 
 		    const bool trace, double floor)
@@ -824,11 +866,11 @@ save_ngram_cstr_bin(const EST_String filename, EST_Ngrammar &n,
     
     if (n.representation() == EST_Ngrammar::dense)
     {
-	for(i=0;i<n.states();i++)
+	for(i=0;i<n.num_states();i++)
 	{
 	    
 	    if ( trace )
-		cerr << "\r" << i*100/n.states() << "%";
+		cerr << "\r" << i*100/n.num_states() << "%";
 	    
 	    for (k=n.p_states[i].pdf().item_start();
 		 !n.p_states[i].pdf().item_end(k);
