@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1997                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
-/*   3. Original authors' names are not delete.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   3. Original authors' names are not deleted.                         */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -39,18 +43,47 @@
 #include "EST_WFST.h"
 #include "wfst_aux.h"
 
+Declare_TList_T(EST_WFST_MultiState *,EST_WFST_MultiStateP)
+
+  // Declare_KVL(int, EST_IList)
+
+	typedef EST_TKVI<int, EST_IList> KVI_int_EST_IList_t; 
+	typedef EST_TKVL<int, EST_IList> KVL_int_EST_IList_t; 
+	
+	static EST_IList int_EST_IList_kv_def_EST_IList_s; 
+	static int int_EST_IList_kv_def_int_s; 
+	
+	EST_IList *EST_TKVL< int, EST_IList >::default_val=&int_EST_IList_kv_def_EST_IList_s; 
+	int *EST_TKVL< int, EST_IList >::default_key=&int_EST_IList_kv_def_int_s; 
+	
+	Declare_TList_N(KVI_int_EST_IList_t, 0)
+
+
 #if defined(INSTANTIATE_TEMPLATES)
 #include "../base_class/EST_TList.cc"
-#include "../base_class/EST_TSortable.cc"
-template class EST_TList<EST_WFST_MultiState *>;
-template class EST_TItem<EST_WFST_MultiState *>;
 
-#include "../base_class/EST_KV.cc"
+  Instantiate_TList_T_MIN(EST_WFST_MultiState *,EST_WFST_MultiStateP)
+
+#include "../base_class/EST_TKVL.cc"
+
+	  //  Instantiate_KVL(int, EST_IList)
+
+        template class EST_TKVL<int, EST_IList>; 
+        template class EST_TKVI<int, EST_IList>; 
+// ostream &operator<<(ostream &s, EST_TKVI< int , EST_IList > const &i){  return s << i.k << "\t" << i.v << "\n"; } 
+//	ostream& operator << (ostream& s,  EST_TKVL< int , EST_IList > const &l) {EST_Litem *p; for (p = l.list.head(); p ; p = next(p)) s << l.list(p).k << "\t" << l.list(p).v << endl; return s;} 
+        Instantiate_TIterator_T(KVL_int_EST_IList_t, KVL_int_EST_IList_t::IPointer_k, int, KVL_int_EST_IList_kitt) 
+        Instantiate_TStructIterator_T(KVL_int_EST_IList_t, KVL_int_EST_IList_t::IPointer, KVI_int_EST_IList_t, KVL_int_EST_IList_itt) 
+        Instantiate_TIterator_T(KVL_int_EST_IList_t, KVL_int_EST_IList_t::IPointer, KVI_int_EST_IList_t, KVL_int_EST_IList_itt) 
+
+	  // Instantiate_TList(KVI_int_EST_IList_t)
+
+	template class EST_TList< TLIST_KVI_int_EST_IList_t_VAL >; 
+	template class EST_TItem< TLIST_KVI_int_EST_IList_t_VAL >; 
+	template const char *error_name(EST_TList< KVI_int_EST_IList_t > val); 
+
+        Instantiate_TIterator_T( EST_TList<KVI_int_EST_IList_t>, EST_TList<KVI_int_EST_IList_t>::IPointer, KVI_int_EST_IList_t, TList_KVI_int_EST_IList_t_itt);
 
-template class KVL<int, EST_IList>;
-template class KVI<int, EST_IList>;
-template class EST_TList<KVI<int, EST_IList> >;
-template class EST_TItem<KVI<int, EST_IList> >;
 
 #endif
 
@@ -292,7 +325,7 @@ void EST_WFST::add_epsilon_reachable(EST_WFST_MultiState *ms) const
 	    {
 		// Add to end of ii if not already there
 		int nstate = s->transitions(i)->state();
-		if (!is_a_member(ii,nstate));
+		if (!is_a_member(ii,nstate))
 		{
 		    ii.append(nstate);
 		    ms->add(nstate); // gets added in order
@@ -466,7 +499,7 @@ void EST_WFST::minimize(const EST_WFST &nmwfst)
     {
 	if (p_states[state_map(i)] == 0)
 	    p_states[state_map(i)] = 
-		copy_and_map_states(state_map,nmwfst.state(i));
+		copy_and_map_states(state_map,nmwfst.state(i),nmwfst);
     }
 
 }
@@ -546,8 +579,32 @@ static int check_distinguished(const EST_WFST &nmwfst,
     }
 }
 
+void EST_WFST::extend_alphabets(const EST_WFST &b)
+{
+    // Extend current in/out alphabets to accomodate anthing in b's
+    // that are not in a's
+    // This guarantees that the number in this will still be valid
+    EST_StrList ivocab, ovocab;
+    int i;
+    
+    for (i=0; i<p_in_symbols.length(); i++)
+ 	ivocab.append(in_symbol(i));
+    for (i=0; i<b.p_in_symbols.length(); i++)
+	if (!strlist_member(ivocab,b.in_symbol(i)))
+	    ivocab.append(b.in_symbol(i));
+    for (i=0; i<p_out_symbols.length(); i++)
+	ovocab.append(out_symbol(i));
+    for (i=0; i<b.p_out_symbols.length(); i++)
+	if (!strlist_member(ovocab,b.out_symbol(i)))
+	    ovocab.append(b.out_symbol(i));
+
+    p_in_symbols.init(ivocab);
+    p_out_symbols.init(ovocab);
+}
+
 EST_WFST_State *EST_WFST::copy_and_map_states(const EST_IVector &state_map,
-					      const EST_WFST_State *s) const
+					      const EST_WFST_State *s,
+					      const EST_WFST &b) const
 {
     // Copy s into new state mapping new states to those in state map
     EST_WFST_State *ns = new EST_WFST_State(state_map(s->name()));
@@ -560,9 +617,9 @@ EST_WFST_State *EST_WFST::copy_and_map_states(const EST_IVector &state_map,
 	int mapped_state= state_map(s->transitions(i)->state());
 	if (mapped_state != WFST_ERROR_STATE)
 	    ns->add_transition(s->transitions(i)->weight(),
-			       mapped_state,
-			       s->transitions(i)->in_symbol(),
-			       s->transitions(i)->out_symbol());
+		    mapped_state,
+		    in_symbol(b.in_symbol(s->transitions(i)->in_symbol())),
+		    out_symbol(b.out_symbol(s->transitions(i)->out_symbol())));
     }
 
     return ns;
@@ -588,6 +645,58 @@ void EST_WFST::complement(const EST_WFST &a)
     }
 }
 
+static int noloopstostart(const EST_WFST &a)
+{
+    // TRUE if there are no transitions leading to the start state
+    // when this is true there is a union operation which preserves
+    // deterministicness
+    int i;
+
+    for (i=0; i < a.num_states(); i++)
+    {
+	const EST_WFST_State *s = a.state(i);
+	EST_Litem *p;
+	for (p=s->transitions.head(); p != 0; p=next(p))
+	{
+	    if (s->transitions(p)->state() == a.start_state())
+		return FALSE;
+	}
+    }
+    return TRUE;
+}
+
+int EST_WFST::deterministiconstartstates(const EST_WFST &a, const EST_WFST &b) const
+{
+    // TRUE if there are no common transition labels from a and b's
+    // start state
+    EST_IMatrix tab;
+    int in,out;
+
+    tab.resize(a.p_in_symbols.length(),a.p_out_symbols.length());
+    tab.fill(0);
+
+    for (EST_Litem *t=a.state(a.start_state())->transitions.head();
+	 t != 0; t=next(t))
+    {
+	tab(a.state(a.start_state())->transitions(t)->in_symbol(),
+	    a.state(a.start_state())->transitions(t)->out_symbol()) = 1;
+    }
+
+    for (EST_Litem *tt=b.state(b.start_state())->transitions.head();
+	 tt != 0; tt=next(tt))
+    {
+	in = a.in_symbol(b.in_symbol(b.state(b.start_state())->transitions(tt)->in_symbol()));
+	out = a.out_symbol(b.out_symbol(b.state(b.start_state())->transitions(tt)->out_symbol()));
+	if (in == -1) 
+	    continue;  // obviously not a clash
+	else if (out == -1) 
+	    continue;  // obviously not a clash
+	else if (tab(in,out) == 1)
+	    return FALSE;
+    }
+    return TRUE;
+}
+
 /***********************************************************************/
 /* Build a WFST which accepts both strings of a and of b               */
 /***********************************************************************/
@@ -596,21 +705,89 @@ void EST_WFST::uunion(const EST_WFST &a,const EST_WFST &b)
     EST_IVector smap;
     int i;
 
+    copy(a);
+    extend_alphabets(b);
+
+    if (a.deterministic() && b.deterministic() &&
+	noloopstostart(a) && noloopstostart(b) &&
+	deterministiconstartstates(a,b))
+    {
+	// This does the union without the epsilon and will preserve
+	// determistic wfsts in this special case
+	smap.resize(b.num_states());
+	smap[0] = start_state();
+	for (i=1; i < b.num_states(); ++i)
+	    smap[i] = i+a.num_states()-1;
+
+	more_states(a.num_states()+b.num_states()-1);
+	p_num_states += b.num_states()-1;
+	for (i=1; i < b.num_states(); i++)
+	    p_states[smap(i)] = copy_and_map_states(smap,b.state(i),b);
+
+	const EST_WFST_State *s = b.state(b.start_state());
+	EST_Litem *p;
+	for (p=s->transitions.head(); p != 0; p=next(p))
+	{
+	    int mapped_state= smap(s->transitions(p)->state());
+	    if (mapped_state != WFST_ERROR_STATE)
+		p_states[start_state()]->
+		 add_transition(s->transitions(p)->weight(),
+		   mapped_state,
+		   in_symbol(b.in_symbol(s->transitions(p)->in_symbol())),
+		   out_symbol(b.out_symbol(s->transitions(p)->out_symbol())));
+	}
+    }
+    else
+    {   // do it the hard way
+	smap.resize(b.num_states());
+	for (i=0; i < b.num_states(); ++i)
+	    smap[i] = i+a.num_states();
+
+	more_states(a.num_states()+b.num_states());
+	p_num_states += b.num_states();
+	for (i=0; i < b.num_states(); i++)
+	    p_states[smap(i)] = copy_and_map_states(smap,b.state(i),b);
+
+	// Actually do the union bit by adding an epsilon transition from
+	// the start of a to start state of b
+	p_states[start_state()]->add_transition(0.0,smap[b.start_state()],
+						in_epsilon(), out_epsilon());
+    }
+
+}
+
+/***********************************************************************/
+/* Build a WFST which a followed by b                                  */
+/***********************************************************************/
+void EST_WFST::concat(const EST_WFST &a,const EST_WFST &b)
+{
+    EST_IVector smap;
+    int i;
+
     copy(a);  
+    extend_alphabets(b);
 
     smap.resize(b.num_states());
     for (i=0; i < b.num_states(); ++i)
 	smap[i] = i+a.num_states();
 
     more_states(a.num_states()+b.num_states());
+
+    // everything final in a becomes non final and an epsilon transition
+    // goes from them to the start of b
+    for (i=0; i < num_states(); i++)
+    {
+	if (p_states[i]->type() == wfst_final)
+	{
+	    p_states[i]->set_type(wfst_nonfinal);
+	    p_states[i]->add_transition(0.0,smap[b.start_state()],
+					in_epsilon(), out_epsilon());
+	}
+    }
+
     p_num_states += b.num_states();
     for (i=0; i < b.num_states(); i++)
-	p_states[smap(i)] = copy_and_map_states(smap,b.state(i));
-
-    // Actually do the union bit by adding an epsilon transition from
-    // the start of a to start state of b
-    p_states[start_state()]->add_transition(0.0,smap[b.start_state()],
-					  in_epsilon(), out_epsilon());
+	p_states[smap(i)] = copy_and_map_states(smap,b.state(i),b);
 
 }
 
@@ -782,10 +959,9 @@ int EST_WFST::can_reach_final(int state)
 /***********************************************************************/
 /* True is wfst is deterministic                                       */
 /***********************************************************************/
-int EST_WFST::deterministic()
+int EST_WFST::deterministic() const
 {
-    // True is all states contains no multiple arcs with the same symbol
-    // can reach a final state.  Mark all others as error states
+    // True if all states contains no multiple arcs with the same symbol
     EST_IMatrix tab;
 
     tab.resize(p_in_symbols.length(),p_out_symbols.length());
