@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -36,15 +40,17 @@
 #ifndef __EST_TVector_H__
 #define __EST_TVector_H__
 
+#include <iostream.h>
 #include "EST_bool.h"
-#include "EST_TList.h"
 #include "EST_rw_status.h"
 
 #include "instantiate/EST_TVectorI.h"
 
 template<class T> class EST_TMatrix;
+template<class T> class EST_TList;
+class EST_String;
 
-/** A constants to make it clearer what is going on when we pass `-1'
+/* A constants to make it clearer what is going on when we pass `-1'
   * meaning `current size' or `all the rest'
   */
 
@@ -53,7 +59,7 @@ extern const int EST_ALL;
 
 /* When set bounds checks (safe but slow) are done on vector access */
 #ifndef TVECTOR_BOUNDS_CHECKING
-#    define TVECTOR_BOUNDS_CHECKING 1
+#    define TVECTOR_BOUNDS_CHECKING 0
 #endif
 
 #if TVECTOR_BOUNDS_CHECKING
@@ -64,10 +70,25 @@ extern const int EST_ALL;
 
 #define INLINE inline
 
+/* This doesn't work as I thought so I have disabled it for now.
+ */
 
-/**  A template vector class. This serves as a base class for a vector
-     of type {\tt T}.  This acts as a higher level version of a normal 
-     C array as defined as {\tt float *x} etc.
+#if defined(__GNUC__) && 0
+#    define fast_a_v_gcc(C) \
+	( *((T *)\
+	      (((char (*) [sizeof(T)*p_column_step])p_memory) + (C))\
+	      ))
+#    define fast_a_v_x(C) (fast_a_v_gcc(C))
+#else
+#    define fast_a_v_x(C) (fast_a_v(C))
+#endif
+
+
+/**@name Template vector
+
+    This serves as a base class for a vector
+     of type <type>T</type>.  This acts as a higher level
+     version of a normal C array as defined as <type>float *x</type> etc.
 
      The vector can be resized after declaration, access can be 
      with or without bounds checking.  Round brackets denote read-only
@@ -76,14 +97,14 @@ extern const int EST_ALL;
 
      The standard operators () and [] should be thought of as 
      having no bounds checking, though they may do so optionally
-     as a compile time option.  The methods {\tt a\_check} and 
-     {\tt a\_nocheck} provide explicit boundary checking/nonchecking,
+     as a compile time option.  The methods <method>a_check</method> and 
+     <method>a_nocheck</method> provide explicit boundary checking/nonchecking,
      both const and non-const versions are provided.
 
      Access through () and [] are guaranteed to be as fast as standard
      C arrays (assuming a resonable optimizing compiler).  
 
-     \begin{verbatim}
+     <programlisting>
      EST_FVector x(10);
      int i;
 
@@ -95,11 +116,11 @@ extern const int EST_ALL;
      for (i=10; i < x.length(); ++i)
         x[i] = sqrt((float)i);
 
-     \end{verbatim}	
+     </programlisting>
 
      To instantialte a template for a a vector of type {FooBar}
-     \begin{verbatim}
-     
+
+     <programlisting>
      #include "../base_class/EST_TVector.cc"
      // If you want List to vector convertion (and defined a TList)
      #include "../base_class/EST_Tvectlist.cc"
@@ -107,24 +128,28 @@ extern const int EST_ALL;
      template class EST_TVector<FooBar>;
      template ostream& operator << 
           (ostream &st, const EST_TVector<FooBar> &v);
-     \end{verbatim}
+     </programlisting>
 
      The EST library already has template vector instantiations for
-     {\tt int}, {\tt float}, {\tt double} and \Ref{EST_String}.  Also
-     types are defined for them in \Ref{EST_types.h} as \Ref{EST_IVector},
-     \Ref{EST_FVector}, \Ref{EST_DVector} and \Ref{EST_StrVector} for 
-     {\tt int}s, {\tt float}s, {\tt doubles}s and \Ref{EST_String}s
-     respectively.
-
-  * @see matrix_example
-  */
+     <type>int</type>, <type>float</type>, <type>double</type> and
+     <docppRef linkend='EST_String'>.  Also types are defined for them
+     in <docppRef linkend='EST_types.h'> as <docppRef
+     linkend='EST_IVector'>, <docppRef linkend='EST_FVector'>,
+     <docppRef linkend='EST_DVector'> and <docppRef
+     linkend='EST_StrVector'> for <type>int</type>s,
+     <type>float</type>s, <type>doubles</type>s and <docppRef
+     linkend='EST_String'>s respectively.
 
+  * @see matrix_example */
+//@{
 template <class T> 
 class EST_TVector 
 {
   // protected:
 public:
-  /// Allocated memory
+  /** Pointer to the start of the vector. 
+    * The start of allocated memory is p_memory-p_offset.
+    */
   T *p_memory; 
 
   /// Visible shape
@@ -138,27 +163,25 @@ public:
 
   
   /// The memory access rule, in one place for easy reference
-  INLINE unsigned int vcell_pos(int c,
-			       int o,
-			       int cs) const
-    {return o + c*cs;}
+  INLINE unsigned int vcell_pos(unsigned int c,
+			        unsigned int cs) const
+    {return cs==1?c:c*cs;}
 
-
-  INLINE unsigned int vcell_pos(int c) const
+  INLINE unsigned int vcell_pos(unsigned int c) const
     {
       return vcell_pos(c, 
-		      p_offset,
 		      p_column_step);
     }
 
-  INLINE unsigned int vcell_pos_1(int c) const
+  INLINE unsigned int vcell_pos_1(unsigned int c) const
     {
       return c;
     }
 
-  /// quick method for returning {\tt x[n]}
-  INLINE const T &fast_a(int c) const { return p_memory[vcell_pos(c)]; }
-  INLINE T &fast_a(int c) { return p_memory[vcell_pos(c)]; }
+  /// quick method for returning \(x[n]\)
+  INLINE const T &fast_a_v(int c) const { return p_memory[vcell_pos(c)]; }
+
+  INLINE T &fast_a_v(int c) { return p_memory[vcell_pos(c)]; }
 
   INLINE const T &fast_a_1(int c) const { return p_memory[vcell_pos_1(c)]; }
   INLINE T &fast_a_1(int c) { return p_memory[vcell_pos_1(c)]; }
@@ -205,11 +228,18 @@ public:
     */
   static T *error_return;
 
-  /** resize vector. If <tt>set=1</tt>, then the current values in
-      the vector are preserved up to the new length <tt>n</tt>. If the
+  /** resize vector. If <expr>set=1</expr>, then the current values in
+      the vector are preserved up to the new length <parameter>n</parameter>. If the
       new length exceeds the old length, the rest of the vector is
-      filled with the <tt>def_val</tt>
+      filled with the <variable>def_val</variable>
   */
+  void resize(int n, int set=1); 
+
+  /** For when you absolutely have to have access to the memory.
+    */
+  const T * memory() const { return p_memory; }
+  T * memory(){ return p_memory; }
+
   /**@name access
     * Basic access methods for vectors.
     */
@@ -223,15 +253,15 @@ public:
   int n() const {return num_columns();}
 
   /// read-only const access operator: without bounds checking
-  INLINE const T &a_no_check(int n) const { return fast_a(n); }
+  INLINE const T &a_no_check(int n) const { return fast_a_v_x(n); }
   /// read/write non-const access operator: without bounds checking
-  INLINE T &a_no_check(int n) { return fast_a(n); }
+  INLINE T &a_no_check(int n) { return fast_a_v_x(n); }
   /// read-only const access operator: without bounds checking
   INLINE const T &a_no_check_1(int n) const { return fast_a_1(n); }
   /// read/write non-const access operator: without bounds checking
   INLINE T &a_no_check_1(int n) { return fast_a_1(n); }
 
-#define pp_a_no_check(V,N) (pp_fast_a(V,N))
+  // #define pp_a_no_check(V,N) (pp_fast_a(V,N))
 
   /// read-only const access operator: with bounds checking
   const T &a_check(int n) const;
@@ -245,8 +275,8 @@ public:
   const T &operator () (int n) const {return A_CHECK(n);}
 
   // PT
-  /// non const access operator: return reference to nth member
-      ///  T &operator () (int n) const {return a(n);}
+  // /// non const access operator: return reference to nth member
+  //  T &operator () (int n) const {return a(n);}
 
   /// read/write non const access operator: return reference to nth member
   T &operator [] (int n) { return A_CHECK(n); }
@@ -259,21 +289,20 @@ public:
   /// assignment operator
   EST_TVector &operator=(const EST_TVector &s);
 
-  void resize(int n, int set=1); 
-
-  /// Fill entire array will value <tt>v</tt>.
+  /// Fill entire array will value <parameter>v</parameter>.
   void fill(const T &v);
 
   /// Fill vector with default value
   void empty() { fill(*def_val); }
 
-  /// Save vector to file <tt>filename</tt>.
+  /// Save vector to file <parameter>filename</parameter>.
   EST_write_status save(const EST_String &filename);
 
   /// is true if vectors are equal size and all elements are equal.
   int operator == (const EST_TVector &v) const;
   /// is true if vectors are not equal size or a single elements isn't equal.
-  int operator != (const EST_TVector &v) const;
+  int operator != (const EST_TVector &v) const
+    { return ! ((*this) == v); }
 
   /// Copy data in and out. Subclassed by SimpleVector for speed.
 
@@ -287,9 +316,13 @@ public:
 
   /// Matrix must be friend to set up subvectors
   friend class EST_TMatrix<T>;
+
+  void integrity() const;
+
 };
 
-/// assignment operator: fill track with values in list <tt>s</tt>.
+//@}
+/// assignment operator: fill track with values in list <parameter>s</paramater>.
 template<class T>
 extern EST_TVector<T> &set(EST_TVector<T> &v, const EST_TList<T> &s);
 
