@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1996,1997                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify and distribute this software and its */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -44,9 +48,7 @@
 #include "siod.h"
 #define wagon_error(WMESS) (cerr << WMESS << endl,exit(-1))
 
-extern char *wagon_version;
-
-// I get floating point exceptions of Alphas when I do any comaprisons
+// I get floating point exceptions of Alphas when I do any comparisons
 // with HUGE_VAL or FLT_MAX so I'll make my own
 #define WGN_HUGE_VAL 1.0e20
 
@@ -54,8 +56,8 @@ class WVector : public EST_FVector
 {
   public:
     WVector(int n) : EST_FVector(n) {}
-    int get_int_val(int n) const { return (int)a_check(n); }
-    float get_flt_val(int n) const { return a_check(n); }
+    int get_int_val(int n) const { return (int)a_no_check(n); }
+    float get_flt_val(int n) const { return a_no_check(n); }
     void set_int_val(int n,int i) { a_check(n) = (int)i; }
     void set_flt_val(int n,float f) { a_check(n) = f; }
 };
@@ -73,7 +75,7 @@ class WDataSet : public WVectorList {
     EST_IVector p_ignore;
     EST_StrVector p_name;
   public:
-    void load_description(const EST_String& descfname);
+    void load_description(const EST_String& descfname,LISP ignores);
 
     int ftype(const int &i) const {return p_type(i);}
     int ignore(int i) const {return p_ignore(i); }
@@ -114,7 +116,6 @@ class WQuestion {
     const wn_oper get_op(void) const {return op;}
     const EST_Val get_operand1(void) const {return operand1;}
     const EST_IList &get_operandl(void) const {return operandl;}
-    const int num_yes(void) const {return yes;}
     const float get_score(void) const {return score;}
     void set_score(const float &f) {score=f;}
     const int ask(const WVector &w) const;
@@ -137,9 +138,9 @@ class WImpurity {
     WImpurity() { t=wnim_unset; a.reset(); }
     WImpurity(const WVectorVector &ds);
     float measure(void);
-    int samples(void);
+    double samples(void);
     wnim_type type(void) const { return t;}
-    void cumulate(const float pv);
+    void cumulate(const float pv,double count=1.0);
     EST_Val value(void);
     EST_DiscreteProbDistribution &pd() { return p; }
     float cluster_distance(int i); // distance i from centre in sds
@@ -199,15 +200,12 @@ class WNode {
     friend ostream& operator<<(ostream &s, WNode &n);
 };
 
+extern Discretes wgn_discretes;
 extern WDataSet wgn_dataset;
 extern WDataSet wgn_test_dataset;
-extern Discretes wgn_discretes;
-extern EST_String wgn_test_file;
-extern EST_String wgn_desc_file;
-extern int wgn_min_cluster_size;
 extern EST_FMatrix wgn_DistMatrix;
 
-void wgn_load_datadescription(EST_String fname);
+void wgn_load_datadescription(EST_String fname,LISP ignores);
 void wgn_load_dataset(WDataSet &ds,EST_String fname);
 WNode *wgn_build_tree(float &score);
 WNode *wgn_build_dlist(float &score,ostream *output);
@@ -217,13 +215,18 @@ void wgn_find_split(WQuestion &q,WVectorVector &ds,
 		WVectorVector &y,WVectorVector &n);
 float summary_results(WNode &tree,ostream *output);
 
+extern int wgn_min_cluster_size;
 extern int wgn_held_out;
 extern int wgn_prune;
 extern int wgn_quiet;
 extern int wgn_verbose;
-extern int wgn_csubset;
+extern int wgn_predictee;
+extern int wgn_count_field;
+extern EST_String wgn_count_field_name;
+extern EST_String wgn_predictee_name;
 extern float wgn_float_range_split;
 extern float wgn_balance;
+extern EST_String wgn_opt_param;
 
 #define wgn_ques_feature(X) (get_c_string(car(X)))
 #define wgn_ques_oper_str(X) (get_c_string(car(cdr(X))))
