@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1995,1996                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -31,7 +35,7 @@
 /**@name EST_track_aux.h
   * EST_Track Auxiliary functions
   * @author Paul Taylor <pault@cstr.ed.ac.uk>
-  * @version $Id: EST_track_aux.h,v 1.1 1998/08/12 09:29:25 awb Exp $
+  * @version $Id: EST_track_aux.h,v 1.2 2001/04/04 13:11:27 awb Exp $
   */
 
 //@{
@@ -41,7 +45,7 @@
 
 #include "EST_FMatrix.h"
 #include "EST_TList.h"
-#include "ling_class/EST_Stream.h"
+#include "ling_class/EST_Relation.h"
 #include "EST_Option.h"
 #include "EST_Track.h"
 #include "EST_TBuffer.h"
@@ -122,7 +126,7 @@ void move_start(EST_Track &tr, float shift);
 EST_Track error(EST_Track &ref, EST_Track &test, int relax= 0);
 void extract(EST_Track &orig, float start, float end, EST_Track &res);
 
-int track_divide(EST_TrackList &mtfr, EST_Track &fv, EST_Stream &key);
+int track_divide(EST_TrackList &mtfr, EST_Track &fv, EST_Relation &key);
 void ParallelTracks(EST_Track &a, EST_TrackList &list,const EST_String &style);
 void track_info(EST_Track &track);
 
@@ -132,6 +136,36 @@ EST_String options_subtrack(void);
 
 int read_track(EST_Track &tr, const EST_String &in_file, EST_Option &al);
 
+/** Return the frame size in {\bf seconds} based on analysis of
+current time points.  This function basically determines the local
+frame size (shift) by subtracting the current time point from the next
+time point. If the {\tt prefer_prev} flag is set to {\tt true}, or the
+index is the last in the track, the size is determined by subtracting
+the previous time point from the current one.
+
+This is most commonly used in pitch synchronous analysis to determine
+the local pitch period.
+
+@see get_frame_size
+*/
+
+float get_time_frame_size(EST_Track &pms, int i, int prefer_prev = 0);
+
+/** Return the frame size in {\bf samples} based on analysis of
+current time points.  This function basically determines the local
+frame size (shift) by subtracting the current time point from the next
+time point. If the {\tt prefer_prev} flag is set to {\tt true}, or the
+index is the last in the track, the size is determined by subtracting
+the previous time point from the current one.
+
+This is most commonly used in pitch synchronous analysis to determine
+the local pitch period.
+
+@see get_time_frame_size
+*/
+int get_frame_size(EST_Track &pms, int current_pos, int sample_rate, 
+			 int prefer_prev=0);
+
 
 /// How many coefficiants in track (looks for Coef0 and coefN channels)
 int get_order(const EST_Track &t, EST_CoefficientType type, int d=0);
@@ -207,10 +241,9 @@ void channel_to_time(EST_Track &tr, const EST_String c_name, float scale=1.0);
 void channel_to_time_lengths(EST_Track &tr, int channel, float scale=1.0);
 void channel_to_time_lengths(EST_Track &tr, EST_ChannelType c,float  scale=1.0);
 void channel_to_time_lengths(EST_Track &tr, const EST_String c_name, float scale=1.0);
-    
 
-// Used when setting Tracks in Features
-void gc_track(void *t);
+/* Allow EST_Track to be used in an EST_Val */
+VAL_REGISTER_CLASS_DCLS(track,EST_Track)
 
 #endif /* __EST_TRACK_AUX_H__ */
 //@}
