@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, and modify this software and its            */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -37,21 +41,20 @@
 #include <fstream.h>*/
 //#include "sigpr/EST_pda.h"
 #include "EST_Track.h"
-#include "EST_Option.h"
+#include "EST_Features.h"
 #include "array_smoother.h"
 #include "EST_math.h"
 
-void smooth_portion(EST_Track &c, EST_Option &op);
+void smooth_portion(EST_Track &c, EST_Features &op);
 static void interp(const EST_Track &c, const EST_Track &speech, int fill,
 		   EST_Track &interp);
-static int parse_ms_list(EST_Option &al, struct Ms_Op *ms);
-
+static int parse_ms_list(EST_Features &al, struct Ms_Op *ms);
 struct Ms_Op *default_ms_op(struct Ms_Op *ms);
 
-void smooth_phrase(EST_Track &fz, EST_Track &speech, EST_Option &op, 
+void smooth_phrase(EST_Track &fz, EST_Track &speech, EST_Features &op, 
 		   EST_Track &smi_fz)
 {
-    int n;
+    int n=0;
     EST_Track sm_fz;
     char nstring[10];
 
@@ -63,11 +66,12 @@ void smooth_phrase(EST_Track &fz, EST_Track &speech, EST_Option &op,
     sm_fz = fz;
     sm_fz.set_channel_name("F0", 0);
 
-    n = (int)(op.fval("window_length", 1) / fz.shift());
+    n = (int)(op.F("window_length") / fz.shift());
     sprintf(nstring, "%d", n);
-    op.override_val("point_window_size", nstring);
+    op.set("point_window_size", nstring);
 
-    smooth_portion(sm_fz, op);
+    if (!op.present("icda_no_smooth"))
+	smooth_portion(sm_fz, op);
 
     if (op.present("icda_no_interp"))
     {
@@ -78,15 +82,15 @@ void smooth_phrase(EST_Track &fz, EST_Track &speech, EST_Option &op,
     int fill = op.present("icda_fi") ? 1 : 0;
     interp(sm_fz, speech, fill, smi_fz); // fill unvoiced region
 
-    n = (int)(op.fval("second_length", 1) / fz.shift());
+    n = (int)(op.F("second_length") / fz.shift());
     sprintf(nstring, "%d", n);
-    op.override_val("point_window_size", nstring);
-    smooth_portion(smi_fz, op);
+    op.set("point_window_size", nstring);
 
-    return;
+    if (!op.present("icda_no_smooth"))
+	smooth_portion(smi_fz, op);
 }
 
-void smooth_portion(EST_Track &c, EST_Option &op)
+void smooth_portion(EST_Track &c, EST_Features &op)
 {
     int i;
     float *a;  // need float * so it can be passed to array_smoother
@@ -97,7 +101,7 @@ void smooth_portion(EST_Track &c, EST_Option &op)
     parse_ms_list(op, ms);
 
     if (op.present("point_window_size"))
-	ms->window_length = op.ival("point_window_size");
+	ms->window_length = op.I("point_window_size");
 
     a = new float[c.num_frames()];
     
@@ -137,9 +141,11 @@ static void interp(const EST_Track &c, const EST_Track &speech, int fill,
     float f = c.shift();
 
     interp = c;  // copy track
+
     if (speech.num_frames() < c.num_frames())
-	interp.resize(speech.num_frames(),interp.num_channels());
-    
+	interp.resize(speech.num_frames(), interp.num_channels());
+
+
     for (i = 1; i < interp.num_frames(); ++i)
     {
 	if ((fill == 1) || (speech.a(i) > 0.5))
@@ -165,22 +171,22 @@ static void interp(const EST_Track &c, const EST_Track &speech, int fill,
     }
 }
 
-int parse_ms_list(EST_Option &al, struct Ms_Op *ms)
+int parse_ms_list(EST_Features &al, struct Ms_Op *ms)
 {
     default_ms_op(ms);
     
     if (al.present("smooth_double"))
-	ms->smooth_double = al.ival("smooth_double");
+	ms->smooth_double = al.I("smooth_double");
     if (al.present( "hanning"))
-	ms->apply_hanning = al.ival("hanning");
+	ms->apply_hanning = al.I("hanning");
     if (al.present("extrapolate"))
-	ms->extrapolate = al.ival("extrapolate");
+	ms->extrapolate = al.I("extrapolate");
     if (al.present("first_length"))
-	ms->first_median = al.ival("first_length");
+	ms->first_median = al.I("first_length");
     if (al.present("second_length"))
-	ms->second_median = al.ival("second_length");
+	ms->second_median = al.I("second_length");
     if (al.present("window_length"))
-	ms->window_length = al.ival("window_length");
+	ms->window_length = al.I("window_length");
 
     return 0;
 }
