@@ -5,15 +5,19 @@
 /*                    Copyright (c) 1994,1995,1996                       */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -41,6 +45,7 @@
 #include "EST_Track.h"
 #include "EST_string_aux.h"
 #include "EST_TrackFile.h"
+#include "EST_error.h"
 
 const int EST_Track::default_sample_rate=16000; // occasionally needed for xmg files
 const float EST_Track::default_frame_shift=0.005; // default frame spacing.
@@ -65,7 +70,11 @@ EST_Track::EST_Track(int n_frames, int n_channels)
     p_times.resize(n_frames);
     p_is_val.resize(n_frames);
     p_channel_names.resize(n_channels);
-    char d = 1;
+
+    p_aux.resize(n_frames, 1);
+    p_aux_names.resize(1);
+
+    char d = 0;
     p_is_val.fill(d);
 }
 
@@ -78,13 +87,14 @@ EST_Track::EST_Track(int n_frames, EST_TrackMap &map)
   p_times.resize(n_frames);
   p_is_val.resize(n_frames);
   p_channel_names.resize(n_channels);
-  char d = 1;
+  char d = 0;
   p_is_val.fill(d);
   assign_map(map);
 }
 
 EST_Track::~EST_Track(void)
 {
+  //  clear_features();
 }
 
 void EST_Track::default_channel_names()
@@ -95,22 +105,23 @@ void EST_Track::default_channel_names()
 
 void EST_Track::default_vals(void)
 {
-    p_equal_space = TRUE;
+    p_equal_space = FALSE;
     p_single_break = FALSE;
-    p_file_type = tff_est_binary;
-    p_contour_type = ct_other;
     p_values.resize(0, 0);
     p_times.resize(0);
     p_is_val.resize(0);
+    p_aux.resize(0, 0);
+    p_aux_names.resize(0);
     p_channel_names.resize(0);
-    p_name = "";
     p_map = NULL;
     p_t_offset=0;
+    
+    init_features();
 }
 
 int EST_Track::track_break(int i) const // does location i hold a break?
 {
-    return (!p_is_val(i));
+    return (p_is_val(i));
 }    
 
 void EST_Track::set_break(int i) // make location i hold a break
@@ -118,34 +129,24 @@ void EST_Track::set_break(int i) // make location i hold a break
     if (i > num_frames())
 	cerr << "Requested setting of break value of the end of the array\n";
     
-    p_is_val[i] = 0;
+    p_is_val[i] = 1;
 }    
 
-void EST_Track::assign_map(EST_TrackMap::P map)
-{
-    p_map = map;
-}
-
-void EST_Track::create_map(EST_ChannelNameMap &names)
-{
-    EST_TrackMap::P map = new EST_TrackMap(EST_TM_REFCOUNTED);
-    
-    for (int i = 0; i < num_channels(); i++)
-    {
-	EST_ChannelType type = names.token(p_channel_names(i));
-	
-	if (type != channel_unknown)
-	  map->set(type, i);
-    }
-    
-    assign_map(map);
-}
 
 void EST_Track::set_value(int i) // make location i hold a value
 {
-    p_is_val[i] = 1;
+    p_is_val[i] = 0;
 }    
 
+const EST_String EST_Track::channel_name(int i, const EST_ChannelNameMap &map, int strings_override) const
+{
+    (void)map;
+    (void)strings_override;
+    return p_channel_names(i);
+}    
+
+/* OLD ENUM VERSION 
+
 const EST_String EST_Track::channel_name(int i, const EST_ChannelNameMap &map, int strings_override) const
 {
     EST_ChannelType type = channel_unknown;
@@ -166,12 +167,17 @@ const EST_String EST_Track::channel_name(int i, const EST_ChannelNameMap &map, i
     else 
 	return "track" + itoString(i);
 }    
-
+*/
 void EST_Track::set_channel_name(const EST_String &fn, int i)
 {
     p_channel_names[i] = fn;
 }    
 
+void EST_Track::set_aux_channel_name(const EST_String &fn, int i)
+{
+    p_aux_names[i] = fn;
+}    
+
 ostream& operator << (ostream& s, const EST_Track &tr)
 {
     int i, j;
@@ -180,6 +186,8 @@ ostream& operator << (ostream& s, const EST_Track &tr)
 	s << tr.t(i);
 	for (j = 0; j < tr.num_channels(); ++j)
 	    s << "\t" << tr(i, j);
+	for (j = 0; j < tr.num_aux_channels(); ++j)
+	    s << "\t" << tr.aux(i, j);
 	s << "\t" << !tr.track_break(i) << endl;
     }
     return s;
@@ -192,40 +200,134 @@ void EST_Track::copy(const EST_Track& a)
     p_times = a.p_times;
     p_is_val = a.p_is_val;
     p_t_offset = a.p_t_offset;
+    p_aux = a.p_aux;
+    p_aux_names = a.p_aux_names;
 }
 
 void EST_Track::copy_setup(const EST_Track& a)
 {
     p_equal_space = a.p_equal_space;
     p_single_break = a.p_single_break;
-    p_file_type = a.p_file_type;
-    p_contour_type = a.p_contour_type;
-    p_name = a.p_name;
     p_channel_names = a.p_channel_names;
     p_map = a.p_map;
+    copy_features(a);
 }    
 
-void EST_Track::resize(int new_num_frames, int new_num_channels)
+void EST_Track::resize(int new_num_frames, int new_num_channels, bool set)
 {
-  int old_num_frames = num_frames();
-  if (new_num_frames<0)
-    new_num_frames = num_frames();
-  if (new_num_channels<0)
-    new_num_channels = num_channels();
+    int old_num_frames = num_frames();
 
-  p_values.resize(new_num_frames, new_num_channels);
-  p_times.resize(new_num_frames);
-  p_is_val.resize(new_num_frames);
-  p_channel_names.resize(new_num_channels);
-  for (int i = old_num_frames; i < new_num_frames; ++i)
-    p_is_val[i] = 1;
+    if (new_num_frames<0)
+	new_num_frames = num_frames();
+
+    if (new_num_channels<0)
+	new_num_channels = num_channels();
+
+    p_channel_names.resize(new_num_channels);
+
+    // this ensures the new channels have a default name
+    if (new_num_channels > num_channels())
+	for (int i = num_channels(); i < new_num_channels; ++i)
+	    set_channel_name("track_" + itoString(i), i);
+
+    p_values.resize(new_num_frames, new_num_channels, set);
+    p_times.resize(new_num_frames, set);
+    p_is_val.resize(new_num_frames, set);
+
+    p_aux.resize(new_num_frames, num_aux_channels(), set);
   
+    // Its important that any new vals get set to 0
+    for (int i = old_num_frames; i < num_frames(); ++i)
+	p_is_val.a_no_check(i) = 0;
+
 }
 
-void EST_Track::resize(int new_num_frames, EST_TrackMap &map)
+static void map_to_channels(EST_StrList &channel_map, 
+		     EST_StrList &channel_names)
 {
-    resize(new_num_frames, map.last_channel()+1);
-    assign_map(map);
+    EST_Litem *p;
+    EST_String b, type, first, last;
+    int n_f, n_l;
+
+    for (p = channel_map.head(); p; p = next(p))
+    {
+	b = channel_map(p);
+	if (b.matches("$", 0))
+	{
+	    // do this backwards as types may have "_" in them
+	    b = b.after("$");
+	    if (!b.contains("-"))
+	    {
+		cerr<<"Ill formed coefficient range in map: " << b << "\n";
+		return;
+	    }
+	    type = b.before("-");
+	    first = b.after("-");
+
+	    if (!first.contains("+"))
+	    {
+		cerr<<"Ill formed coefficient range in map: "<<first<<"\n";
+		return;
+	    }
+
+	    last = first.after("+");
+	    first = first.before("+");
+		
+	    n_f = Stringtoi(first);
+	    n_l = Stringtoi(last);
+
+	    for (int i = n_f; i < n_l; ++i)
+		channel_names.append(type + "_" + itoString(i));
+	    channel_names.append(type + "_N");
+	}
+	else
+	    channel_names.append(b);
+    }
+}
+
+void EST_Track::resize(int new_num_frames, EST_StrList &new_channels, bool set)
+{
+    EST_StrList x;
+    map_to_channels(new_channels, x);
+    int i;
+    EST_Litem *p;
+
+    int new_num_channels;
+    new_num_channels = x.length();
+
+    if (new_num_frames<0)
+	new_num_frames = num_frames();
+
+
+    p_channel_names.resize(new_num_channels);
+    // this ensures the new channels have a default name
+
+    for (i = 0, p = x.head(); p ; p = next(p), ++i)
+	set_channel_name(x(p), i);
+
+    p_values.resize(new_num_frames, new_num_channels, set);
+    p_times.resize(new_num_frames, set);
+    p_is_val.resize(new_num_frames, set);
+  
+//  for (int i = 0; i < new_num_frames; ++i)
+//    p_is_val.a_no_check(i) = 1;
+}
+
+void EST_Track::resize_aux(EST_StrList &new_aux_channels, bool set)
+{
+    int i;
+    EST_Litem *p;
+
+    int new_num_channels;
+    new_num_channels = new_aux_channels.length();
+
+    p_aux_names.resize(new_num_channels);
+
+    // this ensures the new channels have a default name
+    for (i = 0, p = new_aux_channels.head(); p ; p = next(p), ++i)
+	set_aux_channel_name(new_aux_channels(p), i);
+
+    p_aux.resize(num_frames(), new_num_channels, set);
 }
 
 EST_Track& EST_Track::operator+=(const EST_Track &a) // add to existing track
@@ -252,7 +354,7 @@ EST_Track& EST_Track::operator+=(const EST_Track &a) // add to existing track
 	for (k = 0; k < num_channels(); ++k)
 	    p_values.a_no_check(k, j) = a(i, k);
 	p_times[j] = a.t(i);
-	p_is_val[j] = a.val(i);
+	p_is_val[j] = a.p_is_val(i);
     }
     
     return *this;
@@ -302,30 +404,6 @@ int EST_Track::channel_position(const char *name, int offset) const
     return -1;
 }
 
-int EST_Track::channel_position(EST_ChannelType type, int offset) const
-{ 
-    if (p_map!=NULL)
-    {
-	int p = (*p_map)(type);
-	return (p!= NO_SUCH_CHANNEL)?(p+offset): NO_SUCH_CHANNEL;
-    }
-    return channel_position(EST_default_channel_names.name(type), offset);
-}
-
-float &EST_Track::a(int i, EST_ChannelType type, int offset)
-{ 
-    short c = NO_SUCH_CHANNEL;
-    
-    if (p_map!=NULL && ((c = p_map->get(type)) != NO_SUCH_CHANNEL))
-	return p_values.a_no_check(i, c+offset);
-    else
-    {
-	cerr << "no channel '" << EST_default_channel_names.name(type) << "' = " << type << "\n";
-    }
-    
-    return *(p_values.error_return);
-}
-
 float &EST_Track::a(int i, const char *name, int offset)
 { 
     int c;
@@ -338,6 +416,26 @@ float &EST_Track::a(int i, const char *name, int offset)
     return *(p_values.error_return);
 }
 
+EST_Val &EST_Track::aux(int i, const char *name)
+{ 
+    for (int c = 0; c < num_aux_channels(); c++)
+	if (aux_channel_name(c) == name)
+	    return p_aux.a_no_check(i, c);
+    
+    cerr << "no auxiliary channel '" << name << "' found\n";
+    return *(p_aux.error_return);
+}
+
+EST_Val &EST_Track::aux(int i, int c)
+{ 
+    return p_aux(i, c);
+}
+
+EST_Val &EST_Track::aux(int i, int c) const
+{ 
+    return ((EST_Track *)this)->aux(i,c);
+}
+
 #define EPSILON (0.0001)
 
 float &EST_Track::a(float t, int c, EST_InterpType interp)
@@ -373,48 +471,78 @@ float &EST_Track::a(float t, int c, EST_InterpType interp)
     return ia;
 }
 
-float &EST_Track::a(float t, EST_ChannelType type, EST_InterpType interp)
-{ 
-    short c = NO_SUCH_CHANNEL;
-    
-    if (p_map!=NULL && (c = p_map->get(type)) != NO_SUCH_CHANNEL)
-	return a(t, c, interp);
-    else
-    {
-	cerr << "no channel '" << EST_default_channel_names.name(type) << "' = " << type << "\n";
-    }
-    return *(p_values.error_return);
-}
-
 int EST_Track::index(float x) const
 {
-  //   cout << "  look for " << x << "\n";
-
-    for (int i = 1; i < num_frames(); ++i)
-	if (x < p_times(i))
+    if (equal_space())
+    {
+	float s = shift();
+	int f = (int)(x/s+0.5);
+	if (f<0)
+	    f=0;
+	else if (f>= num_frames())
+	    f=num_frames()-1;
+	return f;
+    }
+    else if (num_frames() > 0)
+    {
+	int bst, bmid, bend;
+	bst = 1;
+	bend = num_frames();
+	if (x < p_times.a_no_check(bst))
+	    bmid=bst;
+	if (x >= p_times.a_no_check(bend-1))
+	    bmid=bend-1;
+	else
 	{
-	  //	    cout << "     at " << i << " time " << p_times(i) << "\n";
-	    if (fabs(x - p_times(i)) < fabs(x - p_times(i - 1)))
-		return i;
-	    else
-		return i - 1;
+	    while (1)
+	    {
+		bmid = bst + (bend-bst)/2;
+		if (bst == bmid)
+		    break;
+		else if (x < p_times.a_no_check(bmid))
+		{
+		    if (x >= p_times.a_no_check(bmid-1))
+			break;
+		    bend = bmid;
+		}
+		else
+		    bst = bmid;
+	    }
 	}
+	if (fabs(x - p_times.a_no_check(bmid)) < 
+	    fabs(x - p_times.a_no_check(bmid-1)))
+	    return bmid;
+	else
+	    return bmid - 1;
+    }
     
     return num_frames() -1;
 }
 
 int EST_Track::index_below(float x) const
 {
-    for (int i = 1; i < num_frames(); ++i)
-	if (x < p_times.a_no_check(i))
-	    return i - 1;
-    
-    return num_frames()-1;
+    if (equal_space())
+    {
+	float s = shift();
+	int f = (int)(x/s);
+	if (f<0)
+	    f=0;
+	else if (f>= num_frames())
+	    f=num_frames()-1;
+	return f;
+    }
+    else
+    {
+	for (int i = 1; i < num_frames(); ++i)
+	    if (x <= p_times.a_no_check(i))
+		return i - 1;
+	return num_frames()-1;
+    }
 }
 
 int EST_Track::val(int i) const
 {
-    return p_is_val(i);
+    return !p_is_val(i);
 }
 /*
    "p_equal_space" indicates whether the x-axis values are evenly spaced
@@ -459,11 +587,17 @@ int EST_Track::val(int i) const
 
 float EST_Track::end() const
 {
-    return (p_times(prev_non_break(num_frames())));
+    if (num_frames() == 0)
+	return 0.0;
+    else
+	return (p_times(prev_non_break(num_frames())));
 }
 float EST_Track::start() const
 {
-    return (track_break(0) ? p_times(next_non_break(0)) : p_times(0));
+    if (num_frames() == 0)
+	return 0.0;
+    else
+	return (track_break(0) ? p_times(next_non_break(0)) : p_times(0));
 }
 
 float EST_Track::shift() const
@@ -472,12 +606,8 @@ float EST_Track::shift() const
     int j2 = 0;
     
     if (!p_equal_space)
-    {
-	cerr << "Tried to take shift from non-fixed contour " << p_equal_space 
-	    << endl;
-	return -1.0;
-    }
-    //    cout << "shift\n";
+	EST_error("Tried to take shift from non-fixed contour\n");
+
     do
     {
 	j1 = next_non_break(++j1);
@@ -491,7 +621,8 @@ float EST_Track::shift() const
 	if (num_frames() > 1)
 	    return p_times(1) - p_times(0);
 	else
-	    cerr << "Couldn't determine  shift size\n";
+	    EST_error("Couldn't determine shift size\n");	    
+
     }
     return (p_times(j2) - p_times(j1));
 }
@@ -560,9 +691,9 @@ void EST_Track::sample(float f_interval)
     
     for (i = 0; i < n; ++i)
     {
-	new_is_break[i] = interp_value(new_times(i), f_interval);
+	new_is_break[i] = !interp_value(new_times(i), f_interval);
 	for (j = 0; j < num_channels(); ++j)
-	    new_values(i, j) = new_is_break(i) ? interp_amp(new_times(i), j, f_interval): 0.0;
+	    new_values(i, j) = !new_is_break(i) ? interp_amp(new_times(i), j, f_interval): 0.0;
     }
     
     p_times = new_times;
@@ -659,7 +790,13 @@ float EST_Track::estimate_shift(float x)
 void EST_Track::fill_time(float t, int start)
 {
     for (int i = 0; i < num_frames(); ++i)
-	p_times[i] = t * (float) (i + start);
+	p_times.a_no_check(i) = t * (float) (i + start);
+}
+
+void EST_Track::fill_time(EST_Track &t)
+{
+    for (int i = 0; i < num_frames(); ++i)
+	p_times.a_no_check(i) = t.t(i);
 }
 
 void EST_Track::rm_excess_breaks()
@@ -681,7 +818,7 @@ void EST_Track::rm_excess_breaks()
 	    new_values(j, k) = p_values.a_no_check(i, k);
 	new_times[j] = p_times(i);
 	new_is_break[j] = p_is_val(i);
-	while ((!new_is_break(j)) && (!p_is_val(i + 1)))
+	while ((!new_is_break(j)) && (!val(i + 1)))
 	    ++i;
     }
     p_times = new_times;
@@ -771,10 +908,7 @@ void EST_Track::pad_breaks()
 	return;
     
     if (!p_equal_space)
-    {
-	cerr << "Can only operate on fixed data\n";
-	return;
-    }
+	EST_error("pad_breaks: Can only operate on fixed data\n");
     
     EST_FVector new_times;
     EST_FMatrix new_values;
@@ -901,6 +1035,57 @@ int EST_Track::empty() const
     return 1;			// i.e. true
 }
 
+void EST_Track::channel(EST_FVector &cv, const char * name, int startf, int nf)
+{
+    int n;
+    if ((n = channel_position(name)) == -1)
+    {
+	cerr << "No such channel " << name << endl;
+	return;
+    }
+    channel(cv, n, startf, nf);
+}
+
+void EST_Track::sub_track(EST_Track &st,
+			  int start_frame, int nframes,
+			  const EST_String &start_chan_name, int nchans)
+{
+    int start_chan;
+    if (start_chan_name == "")
+	start_chan = 0;
+
+    if ((start_chan = channel_position(start_chan_name)) == -1)
+	EST_error("sub_track: No such channel %s\n", 
+		  (const char *)start_chan_name);
+
+    sub_track(st, start_frame, nframes, start_chan, nchans);
+}
+
+void EST_Track::sub_track(EST_Track &st,
+			  int start_frame, int nframes,
+			  const EST_String &start_chan_name,
+			  const EST_String &end_chan_name) 
+{
+    int start_chan, end_chan, nchans=0;
+
+    if ((start_chan = channel_position(start_chan_name)) == -1)
+	EST_error("sub_track: No such channel %s\n", 
+		  (const char *)start_chan_name);
+
+    if (end_chan_name == "")
+	nchans = EST_ALL;
+    else
+    {
+	if ((end_chan = channel_position(end_chan_name)) == -1)
+	    EST_error("sub_track: No such channel %s\n", 
+		      (const char*)end_chan_name);
+	else
+	    nchans = end_chan - start_chan + 1;
+    }
+
+    sub_track(st, start_frame, nframes, start_chan, nchans);
+}
+
 void EST_Track::sub_track(EST_Track &st,
 			  int start_frame, int nframes,
 			  int start_chan, int nchans)
@@ -913,28 +1098,27 @@ void EST_Track::sub_track(EST_Track &st,
   if (!bounds_check(*this, start_frame, nframes, start_chan, nchans, 0))
     return;
 
-  p_values.sub_matrix(st.p_values, 
-		      start_frame, nframes,
-		      start_chan, nchans);
+  p_values.sub_matrix(st.p_values, start_frame, nframes, start_chan, nchans);
 
-  p_times.sub_vector(st.p_times,
-		     start_frame, nframes);
+  p_times.sub_vector(st.p_times, start_frame, nframes);
 
-  p_is_val.sub_vector(st.p_is_val,
-		      start_frame, nframes);
+  p_is_val.sub_vector(st.p_is_val, start_frame, nframes);
 
-  p_channel_names.sub_vector(st.p_channel_names, 
-			     start_chan, nchans);
+  p_channel_names.sub_vector(st.p_channel_names,   start_chan, nchans);
+
+  // All auxiliary information is included. These are effectively
+  // pointer statements
+
+  p_aux.sub_matrix(st.p_aux, start_frame, nframes, 0, EST_ALL);
+  p_aux_names.sub_vector(st.p_aux_names, 0, EST_ALL);
 
   st.p_t_offset = p_t_offset;
-  st.p_name = p_name;
-
+  
   st.p_equal_space = p_equal_space;
   st.p_single_break = p_single_break;
+  st.copy_features(*this);
 
-  st.p_contour_type = p_contour_type;
-
-  if (p_map!=NULL)
+  if (p_map!=0)
     st.p_map = new EST_TrackMap(p_map, start_chan, EST_TM_REFCOUNTED);
   else
     st.p_map = NULL;
@@ -966,12 +1150,15 @@ void EST_Track::copy_sub_track(EST_Track &st,
   for (int c=0; c<nchans; c++)
     st.p_channel_names.a(c) = p_channel_names.a(c+start_chan);
 
-  st.p_name = p_name;
+  st.p_aux = p_aux;
+  st.p_aux_names = p_aux_names;
+
   st.p_equal_space = p_equal_space;
   st.p_single_break = p_single_break;
-  st.p_contour_type = p_contour_type;
 
-  if (p_map!=NULL)
+  st.copy_features(*this);
+
+  if (p_map!=0)
     st.p_map = new EST_TrackMap(p_map, start_chan, EST_TM_REFCOUNTED);
   else
     st.p_map = NULL;
@@ -1045,11 +1232,12 @@ EST_read_status EST_Track::load(EST_TokenStream &ts, float ishift)
 	
 	stat = (*l_fun)(ts, *this, ishift);
 	
-	if (stat == read_ok)
-	{
-	    set_file_type(t);
+	if (stat != read_format_error)
+	  {
+	    if (stat == read_ok)
+	      set_file_type(t);
 	    break;
-	}
+	  }
     }
     
     return stat;
@@ -1149,3 +1337,99 @@ EST_read_status EST_Track::load_channel_names(const EST_String filename)
     
     return format_ok;
 }
+
+/* code from here down should be deleted once tracp mapping is modified */
+
+
+float &EST_Track::a(float t, EST_ChannelType type, EST_InterpType interp)
+{ 
+    short c = NO_SUCH_CHANNEL;
+    
+    if (p_map!=0 && (c = p_map->get(type)) != NO_SUCH_CHANNEL)
+	return a(t, c, interp);
+    else
+    {
+	cerr << "no channel '" << EST_default_channel_names.name(type) << "' = " << (int)type << "\n";
+    }
+    return *(p_values.error_return);
+}
+
+
+void EST_Track::assign_map(EST_TrackMap::P map)
+{
+    p_map = map;
+}
+
+void EST_Track::create_map(EST_ChannelNameMap &names)
+{
+    EST_TrackMap::P map = new EST_TrackMap(EST_TM_REFCOUNTED);
+    
+    for (int i = 0; i < num_channels(); i++)
+    {
+	EST_ChannelType type = names.token(p_channel_names(i));
+	
+	if (type != channel_unknown)
+	  map->set(type, i);
+    }
+    
+    assign_map(map);
+}
+
+
+void EST_Track::resize(int new_num_frames, EST_TrackMap &map)
+{
+    resize(new_num_frames, map.last_channel()+1);
+    assign_map(map);
+}
+
+
+
+int EST_Track::channel_position(EST_ChannelType type, int offset) const
+{ 
+    if (p_map!=0)
+    {
+	int p = (*p_map)(type);
+	return (p!= NO_SUCH_CHANNEL)?(p+offset): NO_SUCH_CHANNEL;
+    }
+    return channel_position(EST_default_channel_names.name(type), offset);
+}
+
+float &EST_Track::a(int i, EST_ChannelType type, int offset)
+{ 
+    short c = NO_SUCH_CHANNEL;
+    
+    if (p_map!=0 && ((c = p_map->get(type)) != NO_SUCH_CHANNEL))
+	return p_values.a_no_check(i, c+offset);
+    else
+    {
+	cerr << "no channel '" << EST_default_channel_names.name(type) << "' = " << (int)type << "\n";
+    }
+    
+    return *(p_values.error_return);
+}
+
+EST_Track::IPointer_f::IPointer_f()
+{
+  frame = new EST_Track();
+}
+
+EST_Track::IPointer_f::IPointer_f(const IPointer_f &p)
+{
+  frame=new EST_Track(*(p.frame));
+}
+
+EST_Track::IPointer_f::~IPointer_f()
+{
+  if (frame != NULL)
+    {
+      delete frame;
+      frame=NULL;
+    }
+}
+
+
+#if defined(INSTANTIATE_TEMPLATES)
+
+Instantiate_TIterator_T(EST_Track, EST_Track::IPointer_f, EST_Track, Track_itt)
+
+#endif
