@@ -5,15 +5,19 @@
 /*                         Copyright (c) 1996                            */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify, distribute this software and its    */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -46,22 +50,7 @@
 #include "waveP.h"
 #include "EST_FileType.h"
 
-/*************************************************************************/
-/*                                                                       */
-/* To add a new file format you must:                                    */
-/*  1) add a new enum to EST_file_format_t (in wave_io.h)                */
-/*  2) add a translation to/from strings in str_to_file_format and       */
-/*     file_format_to_str (below)                                        */
-/*  3) Write the functions load_wave_NEWFORMAT and save_wave_NEWFORMAT   */
-/*  4) Add entry to WaveFile_IO_Table at foot of this file               */
-/*                                                                       */
-/* If you wish to write the load and save functions in a different file  */
-/* include the appropriate .h above and write them elsewhere.  You'll    */
-/* probably want to include wave_utils.h and wave_io.h there             */
-/*                                                                       */
-/*************************************************************************/
-
-int def_load_sample_rate = 16000;
+static int def_load_sample_rate = 16000;
 
 /*************************************************************************/
 /*                                                                       */
@@ -77,7 +66,7 @@ static const char *NIST_SIG = "NIST_1A\n   1024\n";
 static const char *NIST_END_SIG = "end_head\n";
 #define NIST_HDR_SIZE 1024
 
-static int nist_get_param_int(char *hdr, char *field, int def_val)
+int nist_get_param_int(const char *hdr, const char *field, int def_val)
 {
     char *p;
     int val;
@@ -93,7 +82,7 @@ static int nist_get_param_int(char *hdr, char *field, int def_val)
 
 }
 
-static char *nist_get_param_str(char *hdr, char *field, char *def_val)
+char *nist_get_param_str(const char *hdr, const char *field, const char *def_val)
 {
     char *p,*val;
     int size;
@@ -108,12 +97,12 @@ static char *nist_get_param_str(char *hdr, char *field, char *def_val)
 	return val;
     }
     else
-	return def_val;
+	return wstrdup(def_val);
 
 
 }
 
-static const char *sample_type_to_nist(enum EST_sample_type_t sample_type)
+const char *sample_type_to_nist(enum EST_sample_type_t sample_type)
 {
     const char *c;
     switch (sample_type) {
@@ -138,7 +127,7 @@ static const char *sample_type_to_nist(enum EST_sample_type_t sample_type)
     return c;
 }
 
-static enum EST_sample_type_t nist_to_sample_type(char *type)
+enum EST_sample_type_t nist_to_sample_type(char *type)
 {
     if ((streq(type,"pcm")) ||
 	(streq(type,"PCM")) ||
@@ -197,8 +186,8 @@ enum EST_read_status load_wave_nist(EST_TokenStream &ts, short **data, int
     sample_coding = nist_get_param_str(header,"sample_coding","pcm");
     if (streq(byte_order,"mu-law"))
     {
-	byte_order = (EST_BIG_ENDIAN ? "10" : "01");
-	sample_coding = "ULAW";
+	byte_order = wstrdup((EST_BIG_ENDIAN ? "10" : "01"));
+	sample_coding = wstrdup("ULAW");
     }
 
     /* code for reading in Tony Robinson's shorten files.
@@ -208,7 +197,7 @@ enum EST_read_status load_wave_nist(EST_TokenStream &ts, short **data, int
        unshortening.
        */
     
-    if (strcmp(sample_coding,"pcm,embedded-shorten-v1.1") == 0)
+    if (streq(sample_coding,"pcm,embedded-shorten-v1.1"))
     {
 	char *tmpfile, *cmdstr;
 	enum EST_read_status rval;
@@ -275,6 +264,8 @@ enum EST_read_status load_wave_nist(EST_TokenStream &ts, short **data, int
     *sample_type = st_short;
     *bo = EST_NATIVE_BO;
     *word_size = 2;
+    wfree(sample_coding);
+    wfree(byte_order);
 
     return format_ok;
 }
@@ -470,6 +461,7 @@ enum EST_read_status load_wave_riff(EST_TokenStream &ts, short **data, int
     if (EST_BIG_ENDIAN) dsize = SWAPINT(dsize);
     ts.fread(&shortdata,2,1);
     if (EST_BIG_ENDIAN) shortdata = SWAPSHORT(shortdata);
+
     switch (shortdata)
     {
 	/* This is a non-proprietary format */
@@ -479,6 +471,9 @@ enum EST_read_status load_wave_riff(EST_TokenStream &ts, short **data, int
     case WAVE_FORMAT_MULAW:
 	actual_sample_type = st_mulaw; break;
     case WAVE_FORMAT_ADPCM:
+	fprintf(stderr, "RIFF file: unsupported proprietary sample format ADPCM\n"); 
+	actual_sample_type = st_short;
+	break;
 	/*	  actual_sample_type = st_adpcm; break; */ /* yes but which adpcm ! */
     case WAVE_FORMAT_ALAW:
     default:
@@ -497,9 +492,11 @@ enum EST_read_status load_wave_riff(EST_TokenStream &ts, short **data, int
     if (EST_BIG_ENDIAN) shortdata = SWAPSHORT(shortdata);
     ts.fread(&shortdata,2,1);
     if (EST_BIG_ENDIAN) shortdata = SWAPSHORT(shortdata);
+
     sample_width = (shortdata+7)/8;
     if ((sample_width == 1) && (actual_sample_type == st_short))
-	actual_sample_type = st_schar; /* oops I meant 8 bit */
+	actual_sample_type = st_uchar; /* oops I meant 8 bit */
+
     ts.seek((dsize-16)+ts.tell());     /* skip rest of header */
     while (1)
     {
@@ -568,9 +565,15 @@ enum EST_write_status save_wave_riff(FILE *fp, const short *data, int offset,
 				     enum EST_sample_type_t sample_type, int bo)   
 {
     (void)bo;
-    char *info;
+    const char *info;
     int data_size, data_int;
     short data_short;
+
+    if (sample_type == st_schar)
+      {
+	EST_warning("RIFF format: Signed 8-bit not allowed by this file format");
+	sample_type=st_uchar; 
+      }
     
     info = "RIFF"; fwrite(info,4,1,fp);
     data_size = num_channels*num_samples*get_word_size(sample_type)+ 8+16+12;
@@ -584,15 +587,15 @@ enum EST_write_status save_wave_riff(FILE *fp, const short *data, int offset,
     fwrite(&data_size,1,4,fp);	/* size of header */
     switch (sample_type)
     {
-    case st_short:
-    case st_schar: data_short = WAVE_FORMAT_PCM; break;
-	       case st_mulaw: data_short = WAVE_FORMAT_MULAW; break;
-	       case st_alaw: data_short = WAVE_FORMAT_ALAW; break;
-	       case st_adpcm: data_short = WAVE_FORMAT_ADPCM; break;
-	       default:
-		   fprintf(stderr,"RIFF format: unsupported data format %d\n",
-			   sample_type);
-		   return misc_write_error;
+    case st_short:  data_short = WAVE_FORMAT_PCM; break;
+    case st_uchar: data_short = WAVE_FORMAT_PCM; break;
+    case st_mulaw: data_short = WAVE_FORMAT_MULAW; break;
+    case st_alaw: data_short = WAVE_FORMAT_ALAW; break;
+    case st_adpcm: data_short = WAVE_FORMAT_ADPCM; break;
+    default:
+      fprintf(stderr,"RIFF format: unsupported data format %d\n",
+	      sample_type);
+      return misc_write_error;
 	       }
     if (EST_BIG_ENDIAN) data_short = SWAPSHORT(data_short);
     fwrite(&data_short,1,2,fp);	/* sample type */
@@ -723,9 +726,9 @@ enum EST_read_status load_wave_aiff(EST_TokenStream &ts, short **data, int
 	    
 	    ts.seek(ssndchunk.offset+(comm_channels*offset)+ts.tell());
 	    if (length == 0)
-		data_length = comm_samples-offset;
-	    else
-		data_length = length;
+ 		data_length = (comm_samples-offset)*comm_channels;
+  	    else
+ 		data_length = length*comm_channels;
 	    file_data = walloc(unsigned char, 
 			       data_length*comm_channels*
 			       get_word_size(actual_sample_type));
@@ -741,7 +744,7 @@ enum EST_read_status load_wave_aiff(EST_TokenStream &ts, short **data, int
 	    
 	    *data = convert_raw_data(file_data,data_length,
 				     actual_sample_type,bo_big);
-	    *num_samples = data_length;
+	    *num_samples = data_length/comm_channels;
 	    *sample_type = st_short;
 	    *word_size = 2;
 	    *bo = EST_NATIVE_BO;
@@ -762,7 +765,7 @@ enum EST_write_status save_wave_aiff(FILE *fp, const short *data, int offset,
 				     enum EST_sample_type_t sample_type, int bo)   
 {
     (void)bo;
-    char *info;
+    const char *info;
     int data_size, data_int;
     unsigned char ieee_ext_buf[10];
     short data_short;
@@ -966,12 +969,15 @@ enum EST_read_status load_wave_snd(EST_TokenStream &ts, short **data, int
     }
     
     *data = convert_raw_data(file_data,data_length,encoding_type,bo_big);
+
+    if (*data == NULL)
+      return read_error;
     
     *num_samples = data_length/ (*num_channels);
     *sample_type = st_short;
     *bo = EST_NATIVE_BO;
     *word_size = 2;
-    return format_ok;
+    return read_ok;
 }
 
 enum EST_write_status save_wave_snd(FILE *fp, const short *data, int offset,
