@@ -5,15 +5,19 @@
 /*                      Copyright (c) 1996,1997                          */
 /*                        All Rights Reserved.                           */
 /*                                                                       */
-/*  Permission to use, copy, modify and distribute this software and its */
-/*  documentation for research, educational and individual use only, is  */
-/*  hereby granted without fee, subject to the following conditions:     */
+/*  Permission is hereby granted, free of charge, to use and distribute  */
+/*  this software and its documentation without restriction, including   */
+/*  without limitation the rights to use, copy, modify, merge, publish,  */
+/*  distribute, sublicense, and/or sell copies of this work, and to      */
+/*  permit persons to whom this work is furnished to do so, subject to   */
+/*  the following conditions:                                            */
 /*   1. The code must retain the above copyright notice, this list of    */
 /*      conditions and the following disclaimer.                         */
 /*   2. Any modifications must be clearly marked as such.                */
 /*   3. Original authors' names are not deleted.                         */
-/*  This software may not be used for commercial purposes without        */
-/*  specific prior written permission from the authors.                  */
+/*   4. The authors' names are not used to endorse or promote products   */
+/*      derived from this software without specific prior written        */
+/*      permission.                                                      */
 /*                                                                       */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
@@ -48,26 +52,30 @@
 #include "EST_Wagon.h"
 #include "EST_math.h"
 
+Discretes wgn_discretes;
+
 WDataSet wgn_dataset;
 WDataSet wgn_test_dataset;
-Discretes wgn_discretes;
-EST_String wgn_test_file;
-EST_String wgn_desc_file;
-int wgn_min_cluster_size = 50;
 EST_FMatrix wgn_DistMatrix;
+
+int wgn_min_cluster_size = 50;
 int wgn_held_out = 0;
 int wgn_prune = TRUE;
 int wgn_quiet = FALSE;
 int wgn_verbose = FALSE;
-int wgn_csubset = FALSE;
+int wgn_count_field = -1;
+EST_String wgn_count_field_name = "";
+int wgn_predictee = 0;
+EST_String wgn_predictee_name = "";
 float wgn_float_range_split = 10;
 float wgn_balance = 0;
+EST_String wgn_opt_param = "";
 
 static float do_summary(WNode &tree,WDataSet &ds,ostream *output);
 static float test_tree_float(WNode &tree,WDataSet &ds,ostream *output);
 static float test_tree_class(WNode &tree,WDataSet &ds,ostream *output);
 static float test_tree_cluster(WNode &tree,WDataSet &dataset, ostream *output);
-static int wagon_split(WNode &node);
+static int wagon_split(int margin,WNode &node);
 static WQuestion find_best_question(WVectorVector &dset);
 static void construct_binary_ques(int feat,WQuestion &test_ques);
 static float construct_float_ques(int feat,WQuestion &ques,WVectorVector &ds);
@@ -75,29 +83,26 @@ static float construct_class_ques(int feat,WQuestion &ques,WVectorVector &ds);
 static void wgn_set_up_data(WVectorVector &data,const WVectorList &ds,int held_out,int in);
 static WNode *wagon_stepwise_find_next_best(float &bscore,int &best_feat);
 
-static int margin;
-char *wagon_version = "1.0.0 beta October 1997";
+Declare_TList_T(WVector *, WVectorP)
 
 Declare_TVector_Base_T(WVector *,NULL,NULL,WVectorP)
 
-
 #if defined(INSTANTIATE_TEMPLATES)
 // Instantiate class
 #include "../base_class/EST_TList.cc"
 #include "../base_class/EST_TVector.cc"
 
-template class EST_TList<WVector *>;
-template class EST_TItem<WVector *>;
+Instantiate_TList_T(WVector *, WVectorP)
 
 Instantiate_TVector(WVector *)
 
 #endif
 
-void wgn_load_datadescription(EST_String fname)
+void wgn_load_datadescription(EST_String fname,LISP ignores)
 {
     // Load field description for a file
-    wgn_dataset.load_description(fname);
-    wgn_test_dataset.load_description(fname);
+    wgn_dataset.load_description(fname,ignores);
+    wgn_test_dataset.load_description(fname,ignores);
 }
 
 void wgn_load_dataset(WDataSet &dataset,EST_String fname)
@@ -122,7 +127,7 @@ void wgn_load_dataset(WDataSet &dataset,EST_String fname)
 	do 
 	{
 	    int type = dataset.ftype(i);
-	    if (type == wndt_float)
+	    if ((type == wndt_float) || (wgn_count_field == i))
 	    {
 		// need to ensure this is not NaN or Infinity
 		float f = atof(ts.get().string());
@@ -195,9 +200,9 @@ float summary_results(WNode &tree,ostream *output)
 
 static float do_summary(WNode &tree,WDataSet &ds,ostream *output)
 {
-    if (wgn_dataset.ftype(0) == wndt_cluster)
+    if (wgn_dataset.ftype(wgn_predictee) == wndt_cluster)
 	return test_tree_cluster(tree,ds,output);
-    else if (wgn_dataset.ftype(0) >= wndt_class)
+    else if (wgn_dataset.ftype(wgn_predictee) >= wndt_class)
 	return test_tree_class(tree,ds,output);
     else
 	return test_tree_float(tree,ds,output);
@@ -207,11 +212,12 @@ WNode *wgn_build_tree(float &score)
 {
     // Build init node and split it while reducing the impurity
     WNode *top = new WNode();
+    int margin = 0;
 
     wgn_set_up_data(top->get_data(),wgn_dataset,wgn_held_out,TRUE);
 
     margin = 0;
-    wagon_split(*top);  // recursively split data;
+    wagon_split(margin,*top);  // recursively split data;
 
     if (wgn_held_out > 0)
     {
@@ -261,28 +267,45 @@ static float test_tree_class(WNode &tree,WDataSet &dataset,ostream *output)
     EST_StrList lex;
     EST_Litem *p;
     EST_String predict,real;
+    WNode *pnode;
+    double H=0,prob;
     int i,type;
-    float correct=0,total=0;
+    float correct=0,total=0, count=0;
 
     for (p=dataset.head(); p != 0; p=next(p))
     {
-	predict = tree.predict((*dataset(p)));
-	type = dataset.ftype(0);
-	real = wgn_discretes[type].name(dataset(p)->get_int_val(0));
+	pnode = tree.predict_node((*dataset(p)));
+	predict = pnode->get_impurity().value();
+	if (wgn_count_field == -1)
+	    count = 1.0;
+	else
+	    count = dataset(p)->get_flt_val(wgn_count_field);
+	prob = pnode->get_impurity().pd().probability(predict);
+	H += (log(prob))*count;
+	type = dataset.ftype(wgn_predictee);
+	real = wgn_discretes[type].name(dataset(p)->get_int_val(wgn_predictee));
 	if (real == predict)
-	    correct++;
-	total++;
+	    correct += count;
+	total += count;
 	pairs.add_item(real,predict,1);
     }
-    for (i=0; i<wgn_discretes[dataset.ftype(0)].length(); i++)
-	lex.append(wgn_discretes[dataset.ftype(0)].name(i));
+    for (i=0; i<wgn_discretes[dataset.ftype(wgn_predictee)].length(); i++)
+	lex.append(wgn_discretes[dataset.ftype(wgn_predictee)].name(i));
 
     const EST_FMatrix &m = confusion(pairs,lex);
     
     if (output != NULL)
+    {
 	print_confusion(m,pairs,lex);  // should be to output not stdout
+	*output << ";; entropy " << (-1*(H/total)) << " perplexity " <<
+	    pow(2.0,(-1*(H/total)));
+    }
 
-    return (float)correct/(float)total;
+    // Minus it so bigger is better 
+    if (wgn_opt_param == "entropy")
+	return -pow(2.0,(-1*(H/total)));
+    else
+	return (float)correct/(float)total;
 }
 
 static float test_tree_cluster(WNode &tree,WDataSet &dataset,ostream *output)
@@ -297,7 +320,7 @@ static float test_tree_cluster(WNode &tree,WDataSet &dataset,ostream *output)
     for (p=dataset.head(); p != 0; p=next(p))
     {
 	leaf = tree.predict_node((*dataset(p)));
-	real = dataset(p)->get_int_val(0);
+	real = dataset(p)->get_int_val(wgn_predictee);
 	meandist += leaf->get_impurity().cluster_distance(real);
 	right_cluster += leaf->get_impurity().in_cluster(real);
 	ranking += leaf->get_impurity().cluster_ranking(real);
@@ -317,7 +340,7 @@ static float test_tree_cluster(WNode &tree,WDataSet &dataset,ostream *output)
 		    << meandist.mean() << endl;
     }
 
-    return 0;  // not sure what this should be
+    return 10000-meandist.mean();  // this doesn't work but I tested it
 }
 
 static float test_tree_float(WNode &tree,WDataSet &dataset,ostream *output)
@@ -327,19 +350,24 @@ static float test_tree_float(WNode &tree,WDataSet &dataset,ostream *output)
     float predict,real;
     EST_SuffStats x,y,xx,yy,xy,se,e;
     double cor,error;
+    double count;
 
     for (p=dataset.head(); p != 0; p=next(p))
     {
 	predict = tree.predict((*dataset(p)));
-	real = dataset(p)->get_flt_val(0);
-	x += predict;
-	y += real;
+	real = dataset(p)->get_flt_val(wgn_predictee);
+	if (wgn_count_field == -1)
+	    count = 1.0;
+	else
+	    count = dataset(p)->get_flt_val(wgn_count_field);
+	x.cumulate(predict,count);
+	y.cumulate(real,count);
 	error = predict-real;
-	se += error*error;
-	e += fabs(error);
-	xx += predict*predict;
-	yy += real*real;
-	xy += predict*real;
+	se.cumulate((error*error),count);
+	e.cumulate(fabs(error),count);
+	xx.cumulate(predict*predict,count);
+	yy.cumulate(real*real,count);
+	xy.cumulate(predict*real,count);
     }
 
     // Pearson's product moment correlation coefficient
@@ -374,11 +402,13 @@ static float test_tree_float(WNode &tree,WDataSet &dataset,ostream *output)
 	    << " (" << ftoString(e.stddev(),4,1) << ")" << endl;
     }
 
-    return cor;  // should really be % variance, I think
-//    return -sqrt(se.mean());  // should really be % variance, I think
+    if (wgn_opt_param == "rmse")
+	return -sqrt(se.mean());  // * -1 so bigger is better 
+    else
+	return cor;  // should really be % variance, I think
 }
 
-static int wagon_split(WNode &node)
+static int wagon_split(int margin, WNode &node)
 {
     // Split given node (if possible)
     WQuestion q;
@@ -387,7 +417,8 @@ static int wagon_split(WNode &node)
     node.set_impurity(WImpurity(node.get_data()));
     q = find_best_question(node.get_data());
 
-    if (q.get_score() < node.get_impurity().measure())
+    if ((q.get_score() < WGN_HUGE_VAL) &&
+	(q.get_score() < node.get_impurity().measure()))
     {
 	// Ok its worth a split
 	l = new WNode();
@@ -403,9 +434,9 @@ static int wagon_split(WNode &node)
 	    cout << q << endl;
 	}
 	margin++;
-	wagon_split(*l);
+	wagon_split(margin,*l);
 	margin++;
-	wagon_split(*r);
+	wagon_split(margin,*r);
 	margin--;
 	return TRUE;
     }
@@ -450,9 +481,10 @@ static WQuestion find_best_question(WVectorVector &dset)
     bscore = tscore = WGN_HUGE_VAL;
     best_ques.set_score(bscore);
     // test each feature with each possible question
-    for (i=1;i < wgn_dataset.width(); i++)
+    for (i=0;i < wgn_dataset.width(); i++)
     {
-	if (wgn_dataset.ignore(i) == TRUE)
+	if ((wgn_dataset.ignore(i) == TRUE) ||
+	    (i == wgn_predictee))
 	    tscore = WGN_HUGE_VAL;     // ignore this feature this time
 	else if (wgn_dataset.ftype(i) == wndt_binary)
 	{
@@ -465,11 +497,14 @@ static WQuestion find_best_question(WVectorVector &dset)
 	}
 	else if (wgn_dataset.ftype(i) == wndt_ignore)
 	    tscore = WGN_HUGE_VAL;    // always ignore this feature
+#if 0
+	// This doesn't work reasonably 
 	else if (wgn_csubset && (wgn_dataset.ftype(i) >= wndt_class))
 	{
 	    wagon_error("subset selection temporarily deleted");
-//	    tscore = construct_class_ques_subset(i,test_ques,dset);
+	    tscore = construct_class_ques_subset(i,test_ques,dset);
 	}
+#endif
 	else if (wgn_dataset.ftype(i) >= wndt_class)
 	    tscore = construct_class_ques(i,test_ques,dset);
 	if (tscore < bscore)
@@ -667,24 +702,37 @@ static float score_question_set(WQuestion &q, WVectorVector &ds, int ignorenth)
     // score this question as a possible split by finding
     // the sum of the impurities when ds is split with this question
     WImpurity y,n;
-    int d;
+    int d, num_yes, num_no;
+    float count;
     WVector *wv;
 
+    num_yes = num_no = 0;
     for (d=0; d < ds.n(); d++)
     {
 	if ((ignorenth < 2) ||
 	    (d%ignorenth != ignorenth-1))
 	{
 	    wv = ds(d);
+	    if (wgn_count_field == -1)
+		count = 1.0;
+	    else 
+		count = (*wv)[wgn_count_field];
+
 	    if (q.ask(*wv) == TRUE)
-		y.cumulate((*wv)[0]);
+	    {
+		num_yes++;
+		y.cumulate((*wv)[wgn_predictee],count);
+	    }
 	    else
-		n.cumulate((*wv)[0]);
+	    {
+		num_no++;
+		n.cumulate((*wv)[wgn_predictee],count);
+	    }
 	}
     }
 
-    q.set_yes(y.samples());
-    q.set_no(n.samples());
+    q.set_yes(num_yes);
+    q.set_no(num_no);
 
     int min_cluster;
 
@@ -717,17 +765,17 @@ WNode *wagon_stepwise(float limit)
     // I put it into wagon itself.
     // This can be pretty computationally intensive
     WNode *best = 0,*new_best = 0;
-    float bscore,best_score = 0;
+    float bscore,best_score = -WGN_HUGE_VAL;
     int best_feat,i;
     int nf = 1;
 
     // Set all features to ignore
-    for (i=1; i < wgn_dataset.width(); i++)
+    for (i=0; i < wgn_dataset.width(); i++)
 	wgn_dataset.set_ignore(i,TRUE);
 
-    for (i=1; i < wgn_dataset.width(); i++)
+    for (i=0; i < wgn_dataset.width(); i++)
     {
-	if (wgn_dataset.ftype(i) == wndt_ignore)
+	if ((wgn_dataset.ftype(i) == wndt_ignore) || (i == wgn_predictee))
 	{
 	    // This skips the round not because this has anything to
 	    // do with this feature being (user specificed) ignored
@@ -737,7 +785,7 @@ WNode *wagon_stepwise(float limit)
 	}
 	new_best = wagon_stepwise_find_next_best(bscore,best_feat);
 
-	if ((bscore - (bscore * (limit/100))) <= best_score)
+	if ((bscore - fabs(bscore * (limit/100))) <= best_score)
 	{
 	    // gone as far as we can
 	    delete new_best;
@@ -769,14 +817,16 @@ static WNode *wagon_stepwise_find_next_best(float &bscore,int &best_feat)
     // Find which of the currently ignored features will best improve
     // the result
     WNode *best = 0;
-    float best_score = 0;
+    float best_score = -WGN_HUGE_VAL;
     int best_new_feat = -1;
     int i;
 
-    for (i=1; i < wgn_dataset.width(); i++)
+    for (i=0; i < wgn_dataset.width(); i++)
     {
 	if (wgn_dataset.ftype(i) == wndt_ignore)
 	    continue; // user wants me to ignore this completely
+	else if (i == wgn_predictee) // can't use the answer
+	    continue;
 	else if (wgn_dataset.ignore(i) == TRUE)
 	{
 	    WNode *current;
