@@ -6,15 +6,19 @@
 ##                         Copyright (c) 1997                            ##
 ##                        All Rights Reserved.                           ##
 ##                                                                       ##
-##  Permission to use, copy, modify, distribute this software and its    ##
-##  documentation for research, educational and individual use only, is  ##
-##  hereby granted without fee, subject to the following conditions:     ##
+##  Permission is hereby granted, free of charge, to use and distribute  ##
+##  this software and its documentation without restriction, including   ##
+##  without limitation the rights to use, copy, modify, merge, publish,  ##
+##  distribute, sublicense, and/or sell copies of this work, and to      ##
+##  permit persons to whom this work is furnished to do so, subject to   ##
+##  the following conditions:                                            ##
 ##   1. The code must retain the above copyright notice, this list of    ##
 ##      conditions and the following disclaimer.                         ##
 ##   2. Any modifications must be clearly marked as such.                ##
 ##   3. Original authors' names are not deleted.                         ##
-##  This software may not be used for commercial purposes without        ##
-##  specific prior written permission from the authors.                  ##
+##   4. The authors' names are not used to endorse or promote products   ##
+##      derived from this software without specific prior written        ##
+##      permission.                                                      ##
 ##                                                                       ##
 ##  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK        ##
 ##  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      ##
@@ -28,7 +32,10 @@
 ##                                                                       ##
 ###########################################################################
 
-CH_TRACK=$TOP/main/ch_track
+CH_TRACK=$TOP/bin/ch_track
+DATA=../lib/example_data
+
+set -x
 
 test_conversions () {
 
@@ -37,7 +44,7 @@ test_conversions () {
 	/bin/rm -f tmp/ch_track.htk tmp/ch_track.track
 
 	# Get htk version with basic header
-	$CH_TRACK -otype htk data/ch_track.htk -o tmp/ch_track.htk || exit 1
+	$CH_TRACK -otype htk $DATA/ch_track.htk -o tmp/ch_track.htk || exit 1
 
 	$CH_TRACK -otype esps tmp/ch_track.htk -o tmp/ch_track.track || exit 1
 	$CH_TRACK -otype htk tmp/ch_track.track -o tmp/ch_track.htk2 || exit 1
@@ -54,7 +61,7 @@ test_conversions () {
 		else echo ch_track htk to ascii to htk: fail
 	fi
 
-	$CH_TRACK -otype htk data/ch_track.htk -o tmp/ch_track.htk || exit 1
+	$CH_TRACK -otype htk $DATA/ch_track.htk -o tmp/ch_track.htk || exit 1
 
 	$CH_TRACK -otype htk_user tmp/ch_track.htk -o tmp/ch_track.track || exit 1
 	$CH_TRACK -otype htk tmp/ch_track.track -o tmp/ch_track.htk2 || exit 1
@@ -67,10 +74,10 @@ test_conversions () {
 test_info ()
 {
   echo info and help >&2
-  $CH_TRACK -info data/ch_track.htk
+  $CH_TRACK -info $DATA/ch_track.htk
   $CH_TRACK -h 
-  $CH_TRACK data/ch_track.htk -otype esps -o tmp/ch_track.esps
-  $CH_TRACK -info data/ch_track.htk tmp/ch_track.esps
+  $CH_TRACK $DATA/ch_track.htk -otype esps -o tmp/ch_track.esps
+  $CH_TRACK -info $DATA/ch_track.htk tmp/ch_track.esps
 }
 
 echo >$OUTPUT
