@@ -5,15 +5,19 @@
 /*                       Copyright (c) 1996,1997                        */
 /*                        All Rights Reserved.                          */
 /*                                                                      */
-/*  Permission to use, copy, modify, distribute this software and its   */
-/*  documentation for research, educational and individual use only, is */
-/*  hereby granted without fee, subject to the following conditions:    */
+/*  Permission is hereby granted, free of charge, to use and distribute */
+/*  this software and its documentation without restriction, including  */
+/*  without limitation the rights to use, copy, modify, merge, publish, */
+/*  distribute, sublicense, and/or sell copies of this work, and to     */
+/*  permit persons to whom this work is furnished to do so, subject to  */
+/*  the following conditions:                                           */
 /*   1. The code must retain the above copyright notice, this list of   */
 /*      conditions and the following disclaimer.                        */
 /*   2. Any modifications must be clearly marked as such.               */
 /*   3. Original authors' names are not deleted.                        */
-/*  This software may not be used for commercial purposes without       */
-/*  specific prior written permission from the authors.                 */
+/*   4. The authors' names are not used to endorse or promote products  */
+/*      derived from this software without specific prior written       */
+/*      permission.                                                     */
 /*                                                                      */
 /*  THE UNIVERSITY OF EDINBURGH AND THE CONTRIBUTORS TO THIS WORK       */
 /*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING     */
@@ -60,6 +64,24 @@ int main()
 	cout << " word was '" << example.at(start_br[1], end_br[1]- start_br[1]) << "'\n";
       }
 
+    cout << "\n";
+
+    // substituting into a string based on a regular expression
+    EST_String source("http://www.cstr.ed.ac.uk/speech_tools");
+    EST_Regex url_re("\\([a-z]*\\)://\\([^/]*\\)\\(.*\\)");
+    EST_String target("protocol=\\1 host=\\2 path=\\3 dummy=\\6");
+
+    cout << "processing '" <<source <<"'\n";
+    if (source.matches(url_re,
+		       0, 
+		       start_br, end_br))
+      {
+	target.subst(source, start_br, end_br);
+	cout << "   gives '" << target <<"'\n";
+      }
+    else
+      cout <<"No match for URL RE\n";
+
     EST_String complex("what if I don't like 'hello world'?");
 
     EST_String quoted(complex.quote_if_needed('\''));
